@@ -1,4 +1,4 @@
-use bson::doc;
+use bson::{doc, RawBsonRef, RawDocument, Timestamp};
 use lazy_static::lazy_static;
 use serde::de::DeserializeOwned;
 
@@ -7,6 +7,13 @@ use std::{collections::HashSet, sync::Arc, time::Instant};
 use super::{session::TransactionState, Client, ClientSession};
 use crate::{
     bson::Document,
+    change_stream::{
+        event::ChangeStreamEvent,
+        session::SessionChangeStream,
+        ChangeStream,
+        ChangeStreamData,
+        WatchArgs,
+    },
     cmap::{
         conn::PinnedConnectionHandle,
         Connection,
@@ -24,16 +31,17 @@ use crate::{
         UNKNOWN_TRANSACTION_COMMIT_RESULT,
     },
     event::command::{CommandFailedEvent, CommandStartedEvent, CommandSucceededEvent},
+    hello::LEGACY_HELLO_COMMAND_NAME_LOWERCASE,
     operation::{
         AbortTransaction,
+        AggregateTarget,
+        ChangeStreamAggregate,
         CommandErrorBody,
-        CommandResponse,
         CommitTransaction,
         Operation,
-        Response,
         Retryability,
     },
-    options::SelectionCriteria,
+    options::{ChangeStreamOptions, SelectionCriteria},
     sdam::{
         HandshakePhase,
         SelectedServer,
@@ -43,6 +51,7 @@ use crate::{
         TransactionSupportStatus,
     },
     selection_criteria::ReadPreference,
+    ClusterTime,
 };
 
 lazy_static! {
@@ -62,7 +71,7 @@ lazy_static! {
     pub(crate) static ref HELLO_COMMAND_NAMES: HashSet<&'static str> = {
         let mut hash_set = HashSet::new();
         hash_set.insert("hello");
-        hash_set.insert("ismaster");
+        hash_set.insert(LEGACY_HELLO_COMMAND_NAME_LOWERCASE);
         hash_set
     };
 }
@@ -139,12 +148,14 @@ impl Client {
     /// Server selection be will performed using the criteria specified on the operation, if any.
     pub(crate) async fn execute_cursor_operation<Op, T>(&self, op: Op) -> Result<Cursor<T>>
     where
-        Op: Operation<O = CursorSpecification<T>>,
-        T: DeserializeOwned + Unpin + Send + Sync,
+        Op: Operation<O = CursorSpecification>,
     {
         Box::pin(async {
             let mut details = self.execute_operation_with_details(op, None).await?;
-            let pinned = self.pin_connection_for_cursor(&mut details.output)?;
+            let pinned = self.pin_connection_for_cursor(
+                &details.output.operation_output,
+                &mut details.output.connection,
+            )?;
             Ok(Cursor::new(
                 self.clone(),
                 details.output.operation_output,
@@ -161,19 +172,17 @@ impl Client {
         session: &mut ClientSession,
     ) -> Result<SessionCursor<T>>
     where
-        Op: Operation<O = CursorSpecification<T>>,
-        T: DeserializeOwned + Unpin + Send + Sync,
+        Op: Operation<O = CursorSpecification>,
     {
         let mut details = self
             .execute_operation_with_details(op, &mut *session)
             .await?;
 
-        let pinned = if let Some(handle) = session.transaction.pinned_connection() {
-            // Cursor operations on a transaction share the same pinned connection.
-            Some(handle.replicate())
-        } else {
-            self.pin_connection_for_cursor(&mut details.output)?
-        };
+        let pinned = self.pin_connection_for_session(
+            &details.output.operation_output,
+            &mut details.output.connection,
+            session,
+        )?;
         Ok(SessionCursor::new(
             self.clone(),
             details.output.operation_output,
@@ -185,20 +194,106 @@ impl Client {
         self.inner.options.load_balanced.unwrap_or(false)
     }
 
-    fn pin_connection_for_cursor<Op, T>(
+    fn pin_connection_for_cursor(
         &self,
-        details: &mut ExecutionOutput<Op>,
-    ) -> Result<Option<PinnedConnectionHandle>>
-    where
-        Op: Operation<O = CursorSpecification<T>>,
-    {
-        if self.is_load_balanced() && details.operation_output.info.id != 0 {
-            Ok(Some(details.connection.pin()?))
+        spec: &CursorSpecification,
+        conn: &mut Connection,
+    ) -> Result<Option<PinnedConnectionHandle>> {
+        if self.is_load_balanced() && spec.info.id != 0 {
+            Ok(Some(conn.pin()?))
         } else {
             Ok(None)
         }
     }
 
+    fn pin_connection_for_session(
+        &self,
+        spec: &CursorSpecification,
+        conn: &mut Connection,
+        session: &mut ClientSession,
+    ) -> Result<Option<PinnedConnectionHandle>> {
+        if let Some(handle) = session.transaction.pinned_connection() {
+            // Cursor operations on a transaction share the same pinned connection.
+            Ok(Some(handle.replicate()))
+        } else {
+            self.pin_connection_for_cursor(spec, conn)
+        }
+    }
+
+    pub(crate) async fn execute_watch<T>(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: Option<ChangeStreamOptions>,
+        target: AggregateTarget,
+        mut resume_data: Option<ChangeStreamData>,
+    ) -> Result<ChangeStream<ChangeStreamEvent<T>>>
+    where
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
+        Box::pin(async {
+            let pipeline: Vec<_> = pipeline.into_iter().collect();
+            let args = WatchArgs {
+                pipeline,
+                target,
+                options,
+            };
+            let mut implicit_session = resume_data
+                .as_mut()
+                .and_then(|rd| rd.implicit_session.take());
+            let op = ChangeStreamAggregate::new(&args, resume_data)?;
+
+            let mut details = self
+                .execute_operation_with_details(op, implicit_session.as_mut())
+                .await?;
+            if let Some(session) = implicit_session {
+                details.implicit_session = Some(session);
+            }
+            let (cursor_spec, cs_data) = details.output.operation_output;
+            let pinned =
+                self.pin_connection_for_cursor(&cursor_spec, &mut details.output.connection)?;
+            let cursor = Cursor::new(self.clone(), cursor_spec, details.implicit_session, pinned);
+
+            Ok(ChangeStream::new(cursor, args, cs_data))
+        })
+        .await
+    }
+
+    pub(crate) async fn execute_watch_with_session<T>(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: Option<ChangeStreamOptions>,
+        target: AggregateTarget,
+        resume_data: Option<ChangeStreamData>,
+        session: &mut ClientSession,
+    ) -> Result<SessionChangeStream<ChangeStreamEvent<T>>>
+    where
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
+        Box::pin(async {
+            let pipeline: Vec<_> = pipeline.into_iter().collect();
+            let args = WatchArgs {
+                pipeline,
+                target,
+                options,
+            };
+            let op = ChangeStreamAggregate::new(&args, resume_data)?;
+
+            let mut details = self
+                .execute_operation_with_details(op, &mut *session)
+                .await?;
+            let (cursor_spec, cs_data) = details.output.operation_output;
+            let pinned = self.pin_connection_for_session(
+                &cursor_spec,
+                &mut details.output.connection,
+                session,
+            )?;
+            let cursor = SessionCursor::new(self.clone(), cursor_spec, pinned);
+
+            Ok(SessionChangeStream::new(cursor, args, cs_data))
+        })
+        .await
+    }
+
     /// Selects a server and executes the given operation on it, optionally using a provided
     /// session. Retries the operation upon failure if retryability is supported.
     async fn execute_operation_with_retry<T: Operation>(
@@ -237,8 +332,16 @@ impl Client {
             Ok(c) => c,
             Err(mut err) => {
                 err.add_labels_and_update_pin(None, &mut session, None)?;
+                if err.is_read_retryable() && self.inner.options.retry_writes != Some(false) {
+                    err.add_label(RETRYABLE_WRITE_ERROR);
+                }
 
-                if err.is_pool_cleared() {
+                let op_retry = match self.get_op_retryability(&op, &session) {
+                    Retryability::Read => err.is_read_retryable(),
+                    Retryability::Write => err.is_write_retryable(),
+                    _ => false,
+                };
+                if err.is_pool_cleared() || op_retry {
                     return self.execute_retry(&mut op, &mut session, None, err).await;
                 } else {
                     return Err(err);
@@ -246,7 +349,7 @@ impl Client {
             }
         };
 
-        let retryability = self.get_retryability(&conn, &op, &session).await?;
+        let retryability = self.get_retryability(&conn, &op, &session)?;
 
         let txn_number = match session {
             Some(ref mut session) => {
@@ -277,6 +380,8 @@ impl Client {
                 connection: conn,
             }),
             Err(mut err) => {
+                err.wire_version = conn.stream_description()?.max_wire_version;
+
                 // Retryable writes are only supported by storage engines with document-level
                 // locking, so users need to disable retryable writes if using mmapv1.
                 if let ErrorKind::Command(ref mut command_error) = *err.kind {
@@ -336,7 +441,7 @@ impl Client {
             Err(_) => return Err(first_error),
         };
 
-        let retryability = self.get_retryability(&conn, op, session).await?;
+        let retryability = self.get_retryability(&conn, op, session)?;
         if retryability == Retryability::None {
             return Err(first_error);
         }
@@ -530,82 +635,79 @@ impl Client {
         let start_time = Instant::now();
         let command_result = match connection.send_raw_command(raw_cmd, request_id).await {
             Ok(response) => {
-                match T::Response::deserialize_response(&response) {
-                    Ok(r) => {
-                        if let (Some(session), Some(ts)) = (session.as_mut(), r.operation_time()) {
-                            session.advance_operation_time(ts);
-                        }
-                        self.update_cluster_time(&r, session).await;
-                        if r.is_success() {
-                            // Retrieve recovery token from successful response.
-                            Client::update_recovery_token(is_sharded, &r, session).await;
+                async fn handle_response<T: Operation>(
+                    client: &Client,
+                    op: &T,
+                    session: &mut Option<&mut ClientSession>,
+                    is_sharded: bool,
+                    response: RawCommandResponse,
+                ) -> Result<RawCommandResponse> {
+                    let raw_doc = RawDocument::from_bytes(response.as_bytes())?;
 
-                            Ok(CommandResult {
-                                raw: response,
-                                deserialized: r.into_body(),
-                            })
-                        } else {
-                            // if command was ok: 0, try to deserialize the command error.
-                            // if that fails, return a generic error.
-                            Err(response
-                                .body::<CommandErrorBody>()
-                                .map(|error_response| error_response.into())
-                                .unwrap_or_else(|e| {
-                                    Error::from(ErrorKind::InvalidResponse {
-                                        message: format!(
-                                            "error deserializing command error: {}",
-                                            e
-                                        ),
-                                    })
-                                }))
-                        }
-                    }
-                    Err(deserialize_error) => {
-                        // if we failed to deserialize the whole response, try deserializing
-                        // a generic command response without the operation's body.
-                        match response.body::<CommandResponse<Option<CommandErrorBody>>>() {
-                            Ok(error_response) => {
-                                if let (Some(session), Some(ts)) =
-                                    (session.as_mut(), error_response.operation_time())
-                                {
-                                    session.advance_operation_time(ts);
-                                }
-                                self.update_cluster_time(&error_response, session).await;
-                                match error_response.body {
-                                    // if the response was ok: 0, return the command error.
-                                    Some(command_error_response)
-                                        if !error_response.is_success() =>
-                                    {
-                                        Err(command_error_response.into())
-                                    }
-                                    // if the response was ok: 0 but we couldnt deserialize the
-                                    // command error,
-                                    // return a generic error indicating so.
-                                    None if !error_response.is_success() => {
-                                        Err(Error::from(ErrorKind::InvalidResponse {
-                                            message: "got command error but failed to deserialize \
-                                                      response"
-                                                .to_string(),
-                                        }))
-                                    }
-                                    // for ok: 1 just return the original deserialization error.
-                                    _ => {
-                                        Client::update_recovery_token(
-                                            is_sharded,
-                                            &error_response,
-                                            session,
-                                        )
-                                        .await;
-                                        Err(deserialize_error)
-                                    }
-                                }
+                    let ok = match raw_doc.get("ok")? {
+                        Some(b) => crate::bson_util::get_int_raw(b).ok_or_else(|| {
+                            ErrorKind::InvalidResponse {
+                                message: format!(
+                                    "expected ok value to be a number, instead got {:?}",
+                                    b
+                                ),
                             }
-                            // We failed to deserialize even that, so just return the original
-                            // deserialization error.
-                            Err(_) => Err(deserialize_error),
+                        })?,
+                        None => {
+                            return Err(ErrorKind::InvalidResponse {
+                                message: "missing 'ok' value in response".to_string(),
+                            }
+                            .into())
                         }
+                    };
+
+                    let cluster_time: Option<ClusterTime> = raw_doc
+                        .get("$clusterTime")?
+                        .and_then(RawBsonRef::as_document)
+                        .map(|d| bson::from_slice(d.as_bytes()))
+                        .transpose()?;
+
+                    let at_cluster_time = op.extract_at_cluster_time(raw_doc)?;
+
+                    client
+                        .update_cluster_time(cluster_time, at_cluster_time, session)
+                        .await;
+
+                    if let (Some(session), Some(ts)) = (
+                        session.as_mut(),
+                        raw_doc
+                            .get("operationTime")?
+                            .and_then(RawBsonRef::as_timestamp),
+                    ) {
+                        session.advance_operation_time(ts);
+                    }
+
+                    if ok == 1 {
+                        if let Some(ref mut session) = session {
+                            if is_sharded && session.in_transaction() {
+                                let recovery_token = raw_doc
+                                    .get("recoveryToken")?
+                                    .and_then(RawBsonRef::as_document)
+                                    .map(|d| bson::from_slice(d.as_bytes()))
+                                    .transpose()?;
+                                session.transaction.recovery_token = recovery_token;
+                            }
+                        }
+
+                        Ok(response)
+                    } else {
+                        Err(response
+                            .body::<CommandErrorBody>()
+                            .map(|error_response| error_response.into())
+                            .unwrap_or_else(|e| {
+                                Error::from(ErrorKind::InvalidResponse {
+                                    message: format!("error deserializing command error: {}", e),
+                                })
+                            }))
                     }
                 }
+
+                handle_response(self, op, session, is_sharded, response).await
             }
             Err(err) => Err(err),
         };
@@ -642,7 +744,6 @@ impl Client {
                         Document::new()
                     } else {
                         response
-                            .raw
                             .body()
                             .unwrap_or_else(|e| doc! { "deserialization error": e.to_string() })
                     };
@@ -658,7 +759,7 @@ impl Client {
                     handler.handle_command_succeeded_event(command_succeeded_event);
                 });
 
-                match op.handle_response(response.deserialized, connection.stream_description()?) {
+                match op.handle_response(response, connection.stream_description()?) {
                     Ok(response) => Ok(response),
                     Err(mut err) => {
                         err.add_labels_and_update_pin(
@@ -732,67 +833,70 @@ impl Client {
     }
 
     /// Returns the retryability level for the execution of this operation.
-    async fn get_retryability<T: Operation>(
+    fn get_op_retryability<T: Operation>(
+        &self,
+        op: &T,
+        session: &Option<&mut ClientSession>,
+    ) -> Retryability {
+        if session
+            .as_ref()
+            .map(|session| session.in_transaction())
+            .unwrap_or(false)
+        {
+            return Retryability::None;
+        }
+        match op.retryability() {
+            Retryability::Read if self.inner.options.retry_reads != Some(false) => {
+                Retryability::Read
+            }
+            // commitTransaction and abortTransaction should be retried regardless of the
+            // value for retry_writes set on the Client
+            Retryability::Write
+                if op.name() == CommitTransaction::NAME
+                    || op.name() == AbortTransaction::NAME
+                    || self.inner.options.retry_writes != Some(false) =>
+            {
+                Retryability::Write
+            }
+            _ => Retryability::None,
+        }
+    }
+
+    /// Returns the retryability level for the execution of this operation on this connection.
+    fn get_retryability<T: Operation>(
         &self,
         conn: &Connection,
         op: &T,
         session: &Option<&mut ClientSession>,
     ) -> Result<Retryability> {
-        if !session
-            .as_ref()
-            .map(|session| session.in_transaction())
-            .unwrap_or(false)
-        {
-            match op.retryability() {
-                Retryability::Read if self.inner.options.retry_reads != Some(false) => {
-                    return Ok(Retryability::Read);
-                }
-                Retryability::Write if conn.stream_description()?.supports_retryable_writes() => {
-                    // commitTransaction and abortTransaction should be retried regardless of the
-                    // value for retry_writes set on the Client
-                    if op.name() == CommitTransaction::NAME
-                        || op.name() == AbortTransaction::NAME
-                        || self.inner.options.retry_writes != Some(false)
-                    {
-                        return Ok(Retryability::Write);
-                    }
-                }
-                _ => {}
+        match self.get_op_retryability(op, session) {
+            Retryability::Read => Ok(Retryability::Read),
+            Retryability::Write if conn.stream_description()?.supports_retryable_writes() => {
+                Ok(Retryability::Write)
             }
+            _ => Ok(Retryability::None),
         }
-        Ok(Retryability::None)
     }
 
-    async fn update_cluster_time<T: Response>(
+    async fn update_cluster_time(
         &self,
-        command_response: &T,
+        cluster_time: Option<ClusterTime>,
+        at_cluster_time: Option<Timestamp>,
         session: &mut Option<&mut ClientSession>,
     ) {
-        if let Some(cluster_time) = command_response.cluster_time() {
+        if let Some(ref cluster_time) = cluster_time {
             self.inner.topology.advance_cluster_time(cluster_time).await;
             if let Some(ref mut session) = session {
                 session.advance_cluster_time(cluster_time)
             }
         }
 
-        if let Some(timestamp) = command_response.at_cluster_time() {
+        if let Some(timestamp) = at_cluster_time {
             if let Some(ref mut session) = session {
                 session.snapshot_time = Some(timestamp);
             }
         }
     }
-
-    async fn update_recovery_token<T: Response>(
-        is_sharded: bool,
-        response: &T,
-        session: &mut Option<&mut ClientSession>,
-    ) {
-        if let Some(ref mut session) = session {
-            if is_sharded && session.in_transaction() {
-                session.transaction.recovery_token = response.recovery_token().cloned();
-            }
-        }
-    }
 }
 
 async fn get_connection<T: Operation>(
@@ -888,11 +992,6 @@ impl Error {
     }
 }
 
-struct CommandResult<T> {
-    raw: RawCommandResponse,
-    deserialized: T,
-}
-
 struct ExecutionDetails<T: Operation> {
     output: ExecutionOutput<T>,
     implicit_session: Option<ClientSession>,
