@@ -1,4 +1,4 @@
-#[cfg(all(test, not(feature = "sync")))]
+#[cfg(all(test, not(feature = "sync"), not(feature = "tokio-sync")))]
 mod test;
 
 mod resolver_config;
@@ -7,9 +7,7 @@ use std::{
     collections::HashSet,
     convert::TryFrom,
     fmt::{self, Display, Formatter},
-    fs::File,
     hash::{Hash, Hasher},
-    io::{BufReader, Seek, SeekFrom},
     path::PathBuf,
     str::FromStr,
     sync::Arc,
@@ -18,15 +16,6 @@ use std::{
 
 use derivative::Derivative;
 use lazy_static::lazy_static;
-use rustls::{
-    internal::pemfile,
-    Certificate,
-    RootCertStore,
-    ServerCertVerified,
-    ServerCertVerifier,
-    TLSError,
-};
-use rustls_pemfile::{read_one, Item};
 use serde::{
     de::{Error, Unexpected},
     Deserialize,
@@ -36,7 +25,6 @@ use serde::{
 use serde_with::skip_serializing_none;
 use strsim::jaro_winkler;
 use typed_builder::TypedBuilder;
-use webpki_roots::TLS_SERVER_ROOTS;
 
 #[cfg(test)]
 use crate::srv::LookupHosts;
@@ -54,6 +42,9 @@ use crate::{
     srv::{OriginalSrvInfo, SrvResolver},
 };
 
+#[cfg(any(feature = "sync", feature = "tokio-sync"))]
+use crate::runtime;
+
 pub use resolver_config::ResolverConfig;
 
 const DEFAULT_PORT: u16 = 27017;
@@ -244,7 +235,7 @@ impl ServerAddress {
         })
     }
 
-    #[cfg(all(test, not(feature = "sync")))]
+    #[cfg(all(test, not(feature = "sync"), not(feature = "tokio-sync")))]
     pub(crate) fn into_document(self) -> Document {
         match self {
             Self::Tcp { host, port } => {
@@ -322,8 +313,8 @@ impl<'de> Deserialize<'de> for ServerApiVersion {
     }
 }
 
-/// Options used to declare a versioned server API.  For more information, see the [Versioned API](
-/// https://docs.mongodb.com/v5.0/reference/versioned-api/) manual page.
+/// Options used to declare a stable server API.  For more information, see the [Stable API](
+/// https://docs.mongodb.com/v5.0/reference/stable-api/) manual page.
 #[serde_with::skip_serializing_none]
 #[derive(Clone, Debug, Deserialize, Serialize, PartialEq, TypedBuilder)]
 #[builder(field_defaults(setter(into)))]
@@ -419,8 +410,7 @@ pub struct ClientOptions {
     #[builder(default)]
     pub driver_info: Option<DriverInfo>,
 
-    /// The amount of time each monitoring thread should wait between sending an isMaster command
-    /// to its respective server.
+    /// The amount of time each monitoring thread should wait between performing server checks.
     ///
     /// The default value is 10 seconds.
     #[builder(default)]
@@ -506,13 +496,13 @@ pub struct ClientOptions {
     /// The declared API version is applied to all commands run through the client, including those
     /// sent through any handle derived from the client.
     ///
-    /// Specifying versioned API options in the command document passed to `run_command` AND
+    /// Specifying stable API options in the command document passed to `run_command` AND
     /// declaring an API version on the client is not supported and is considered undefined
     /// behaviour. To run any command with a different API version or without declaring one, create
     /// a separate client that declares the appropriate API version.
     ///
-    /// For more information, see the [Versioned API](
-    /// https://docs.mongodb.com/v5.0/reference/versioned-api/) manual page.
+    /// For more information, see the [Stable API](
+    /// https://docs.mongodb.com/v5.0/reference/stable-api/) manual page.
     #[builder(default)]
     pub server_api: Option<ServerApi>,
 
@@ -583,9 +573,6 @@ pub(crate) struct TestOptions {
 
     /// Mock response for `SrvPollingMonitor::lookup_hosts`.
     pub(crate) mock_lookup_hosts: Option<Result<LookupHosts>>,
-
-    /// Mock the `serviceId` response for a load-balanced hello.
-    pub(crate) mock_service_id: bool,
 }
 
 fn default_hosts() -> Vec<ServerAddress> {
@@ -788,99 +775,17 @@ pub struct TlsOptions {
     /// [`Client`](../struct.Client.html) will not attempt to verify its identity to the
     /// server.
     pub cert_key_file_path: Option<PathBuf>,
-}
 
-struct NoCertVerifier {}
-
-impl ServerCertVerifier for NoCertVerifier {
-    fn verify_server_cert(
-        &self,
-        _: &RootCertStore,
-        _: &[Certificate],
-        _: webpki::DNSNameRef,
-        _: &[u8],
-    ) -> std::result::Result<ServerCertVerified, TLSError> {
-        Ok(ServerCertVerified::assertion())
-    }
+    /// Whether or not the [`Client`](../struct.Client.html) should return an error if the hostname
+    /// is invalid.
+    ///
+    /// The default value is to error on invalid hostnames.
+    #[cfg(any(feature = "openssl-tls", docsrs))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "openssl-tls")))]
+    pub allow_invalid_hostnames: Option<bool>,
 }
 
 impl TlsOptions {
-    /// Converts `TlsOptions` into a rustls::ClientConfig.
-    pub(crate) fn into_rustls_config(self) -> Result<rustls::ClientConfig> {
-        let mut config = rustls::ClientConfig::new();
-
-        if let Some(true) = self.allow_invalid_certificates {
-            config
-                .dangerous()
-                .set_certificate_verifier(Arc::new(NoCertVerifier {}));
-        }
-
-        let mut store = RootCertStore::empty();
-        if let Some(path) = self.ca_file_path {
-            store
-                .add_pem_file(&mut BufReader::new(File::open(&path)?))
-                .map_err(|_| ErrorKind::InvalidTlsConfig {
-                    message: format!(
-                        "Unable to parse PEM-encoded root certificate from {}",
-                        path.display()
-                    ),
-                })?;
-        } else {
-            store.add_server_trust_anchors(&TLS_SERVER_ROOTS);
-        }
-
-        config.root_store = store;
-
-        if let Some(path) = self.cert_key_file_path {
-            let mut file = BufReader::new(File::open(&path)?);
-            let certs = match pemfile::certs(&mut file) {
-                Ok(certs) => certs,
-                Err(()) => {
-                    return Err(ErrorKind::InvalidTlsConfig {
-                        message: format!(
-                            "Unable to parse PEM-encoded client certificate from {}",
-                            path.display()
-                        ),
-                    }
-                    .into())
-                }
-            };
-
-            file.seek(SeekFrom::Start(0))?;
-            let key = loop {
-                match read_one(&mut file) {
-                    Ok(Some(Item::PKCS8Key(bytes))) | Ok(Some(Item::RSAKey(bytes))) => {
-                        break rustls::PrivateKey(bytes)
-                    }
-                    Ok(Some(_)) => continue,
-                    Ok(None) => {
-                        return Err(ErrorKind::InvalidTlsConfig {
-                            message: format!("No PEM-encoded keys in {}", path.display()),
-                        }
-                        .into())
-                    }
-                    Err(_) => {
-                        return Err(ErrorKind::InvalidTlsConfig {
-                            message: format!(
-                                "Unable to parse PEM-encoded item from {}",
-                                path.display()
-                            ),
-                        }
-                        .into())
-                    }
-                }
-            };
-
-            config
-                .set_single_client_cert(certs, key)
-                .map_err(|e| ErrorKind::InvalidTlsConfig {
-                    message: e.to_string(),
-                })?;
-        }
-
-        Ok(config)
-    }
-
     #[cfg(test)]
     pub(crate) fn serialize_for_client_options<S>(
         tls_options: &TlsOptions,
@@ -1041,17 +946,17 @@ impl ClientOptions {
     ///
     /// Note: if the `sync` feature is enabled, then this method will be replaced with [the sync
     /// version](#method.parse-1).
-    #[cfg(not(feature = "sync"))]
+    #[cfg(all(not(feature = "sync"), not(feature = "tokio-sync")))]
     pub async fn parse(s: impl AsRef<str>) -> Result<Self> {
         Self::parse_uri(s, None).await
     }
 
     /// This method will be present if the `sync` feature is enabled. It's otherwise identical to
     /// [the async version](#method.parse)
-    #[cfg(any(feature = "sync", docsrs))]
-    #[cfg_attr(docsrs, doc(cfg(feature = "sync")))]
+    #[cfg(any(feature = "sync", feature = "tokio-sync", docsrs))]
+    #[cfg_attr(docsrs, doc(cfg(any(feature = "sync", feature = "tokio-sync"))))]
     pub fn parse(s: impl AsRef<str>) -> Result<Self> {
-        crate::RUNTIME.block_on(Self::parse_uri(s.as_ref(), None))
+        runtime::block_on(Self::parse_uri(s.as_ref(), None))
     }
 
     /// Parses a MongoDB connection string into a `ClientOptions` struct.
@@ -1068,7 +973,7 @@ impl ClientOptions {
     ///
     /// Note: if the `sync` feature is enabled, then this method will be replaced with [the sync
     /// version](#method.parse_with_resolver_config-1).
-    #[cfg(not(feature = "sync"))]
+    #[cfg(all(not(feature = "sync"), not(feature = "tokio-sync")))]
     pub async fn parse_with_resolver_config(
         uri: impl AsRef<str>,
         resolver_config: ResolverConfig,
@@ -1078,10 +983,10 @@ impl ClientOptions {
 
     /// This method will be present if the `sync` feature is enabled. It's otherwise identical to
     /// [the async version](#method.parse_with_resolver_config)
-    #[cfg(any(feature = "sync", docsrs))]
-    #[cfg_attr(docsrs, doc(cfg(feature = "sync")))]
+    #[cfg(any(feature = "sync", feature = "tokio-sync", docsrs))]
+    #[cfg_attr(docsrs, doc(cfg(any(feature = "sync", feature = "tokio-sync"))))]
     pub fn parse_with_resolver_config(uri: &str, resolver_config: ResolverConfig) -> Result<Self> {
-        crate::RUNTIME.block_on(Self::parse_uri(uri, Some(resolver_config)))
+        runtime::block_on(Self::parse_uri(uri, Some(resolver_config)))
     }
 
     /// Populate this `ClientOptions` from the given URI, optionally using the resolver config for
@@ -2018,7 +1923,7 @@ impl ClientOptionsParser {
     }
 }
 
-#[cfg(all(test, not(feature = "sync")))]
+#[cfg(all(test, not(feature = "sync"), not(feature = "tokio-sync")))]
 mod tests {
     use std::time::Duration;
 
@@ -2334,10 +2239,6 @@ mod tests {
         );
     }
 
-    #[cfg_attr(feature = "tokio-runtime", tokio::test)]
-    #[cfg_attr(feature = "async-std-runtime", async_std::test)]
-    async fn with_invalid_read_preference_mode() {}
-
     #[cfg_attr(feature = "tokio-runtime", tokio::test)]
     #[cfg_attr(feature = "async-std-runtime", async_std::test)]
     async fn with_mixed_options() {
