@@ -8,6 +8,7 @@ use std::{
 };
 
 use derivative::Derivative;
+use tokio::sync::{mpsc, Mutex};
 
 use self::wire::Message;
 use super::manager::PoolManager;
@@ -17,7 +18,8 @@ use crate::{
         options::{ConnectionOptions, StreamOptions},
         PoolGeneration,
     },
-    error::{load_balanced_mode_mismatch, ErrorKind, Result},
+    compression::Compressor,
+    error::{load_balanced_mode_mismatch, Error, ErrorKind, Result},
     event::cmap::{
         CmapEventHandler,
         ConnectionCheckedInEvent,
@@ -75,6 +77,18 @@ pub(crate) struct Connection {
 
     stream: AsyncStream,
 
+    /// Compressor that the client will use before sending messages.
+    /// This compressor does not get used to decompress server messages.
+    /// The client will decompress server messages using whichever compressor
+    /// the server indicates in its message.  This compressor is the first
+    /// compressor in the client's compressor list that also appears in the
+    /// server's compressor list.
+    pub(super) compressor: Option<Compressor>,
+
+    /// If the connection is pinned to a cursor or transaction, the channel sender to return this
+    /// connection to the pin holder.
+    pinned_sender: Option<mpsc::Sender<Connection>>,
+
     #[derivative(Debug = "ignore")]
     handler: Option<Arc<dyn CmapEventHandler>>,
 }
@@ -103,6 +117,8 @@ impl Connection {
             handler: options.and_then(|options| options.event_handler),
             stream_description: None,
             error: false,
+            pinned_sender: None,
+            compressor: None,
         };
 
         Ok(conn)
@@ -161,6 +177,10 @@ impl Connection {
         }
     }
 
+    pub(crate) fn service_id(&self) -> Option<ObjectId> {
+        self.generation.service_id()
+    }
+
     pub(crate) fn address(&self) -> &ServerAddress {
         &self.address
     }
@@ -234,9 +254,24 @@ impl Connection {
         }
     }
 
-    async fn send_message(&mut self, message: Message) -> Result<RawCommandResponse> {
+    async fn send_message(
+        &mut self,
+        message: Message,
+        to_compress: bool,
+    ) -> Result<RawCommandResponse> {
         self.command_executing = true;
-        let write_result = message.write_to(&mut self.stream).await;
+
+        // If the client has agreed on a compressor with the server, and the command
+        // is the right type of command, then compress the message.
+        let write_result = match self.compressor {
+            Some(ref compressor) if to_compress => {
+                message
+                    .write_compressed_to(&mut self.stream, compressor)
+                    .await
+            }
+            _ => message.write_to(&mut self.stream).await,
+        };
+
         self.error = write_result.is_err();
         write_result?;
 
@@ -257,8 +292,9 @@ impl Connection {
         command: Command,
         request_id: impl Into<Option<i32>>,
     ) -> Result<RawCommandResponse> {
+        let to_compress = command.should_compress();
         let message = Message::with_command(command, request_id.into())?;
-        self.send_message(message).await
+        self.send_message(message, to_compress).await
     }
 
     /// Executes a `RawCommand` and returns a `CommandResponse` containing the result from the
@@ -272,8 +308,9 @@ impl Connection {
         command: RawCommand,
         request_id: impl Into<Option<i32>>,
     ) -> Result<RawCommandResponse> {
+        let to_compress = command.should_compress();
         let message = Message::with_raw_command(command, request_id.into());
-        self.send_message(message).await
+        self.send_message(message, to_compress).await
     }
 
     /// Gets the connection's StreamDescription.
@@ -286,6 +323,28 @@ impl Connection {
         })
     }
 
+    /// Pin the connection, removing it from the normal connection pool.
+    pub(crate) fn pin(&mut self) -> Result<PinnedConnectionHandle> {
+        if self.pinned_sender.is_some() {
+            return Err(Error::internal(format!(
+                "cannot pin an already-pinned connection (id = {})",
+                self.id
+            )));
+        }
+        if self.pool_manager.is_none() {
+            return Err(Error::internal(format!(
+                "cannot pin a checked-in connection (id = {})",
+                self.id
+            )));
+        }
+        let (tx, rx) = mpsc::channel(1);
+        self.pinned_sender = Some(tx);
+        Ok(PinnedConnectionHandle {
+            id: self.id,
+            receiver: Arc::new(Mutex::new(rx)),
+        })
+    }
+
     /// Close this connection, emitting a `ConnectionClosedEvent` with the supplied reason.
     pub(super) fn close_and_drop(mut self, reason: ConnectionClosedReason) {
         self.close(reason);
@@ -313,6 +372,8 @@ impl Connection {
             error: self.error,
             pool_manager: None,
             ready_and_available_time: None,
+            pinned_sender: self.pinned_sender.clone(),
+            compressor: self.compressor.clone(),
         }
     }
 }
@@ -330,8 +391,41 @@ impl Drop for Connection {
         // the `close_and_drop` helper explicitly, so we don't add it back to the
         // pool or emit any events.
         if let Some(pool_manager) = self.pool_manager.take() {
-            let dropped_connection = self.take();
-            if let Err(mut conn) = pool_manager.check_in(dropped_connection) {
+            let mut dropped_connection = self.take();
+            let result = if let Some(sender) = self.pinned_sender.as_mut() {
+                // Preserve the pool manager and timestamp for pinned connections.
+                dropped_connection.pool_manager = Some(pool_manager.clone());
+                dropped_connection.ready_and_available_time = self.ready_and_available_time;
+                match sender.try_send(dropped_connection) {
+                    Ok(()) => Ok(()),
+                    // The connection has been unpinned and should be checked back in.
+                    Err(mpsc::error::TrySendError::Closed(mut conn)) => {
+                        conn.pinned_sender = None;
+                        conn.ready_and_available_time = None;
+                        pool_manager.check_in(conn)
+                    }
+                    // The connection is being returned to the pin holder while another connection
+                    // is in the pin buffer; this should never happen.  Only possible action is to
+                    // check the connection back in.
+                    Err(mpsc::error::TrySendError::Full(mut conn)) => {
+                        // Panic in debug mode
+                        if cfg!(debug_assertions) {
+                            panic!(
+                                "buffer full when attempting to return a pinned connection (id = \
+                                 {})",
+                                conn.id
+                            );
+                        }
+                        // TODO RUST-230 log an error in non-debug mode.
+                        conn.pinned_sender = None;
+                        conn.ready_and_available_time = None;
+                        pool_manager.check_in(conn)
+                    }
+                }
+            } else {
+                pool_manager.check_in(dropped_connection)
+            };
+            if let Err(mut conn) = result {
                 // the check in failed because the pool has been dropped, so we emit the event
                 // here and drop the connection.
                 conn.close(ConnectionClosedReason::PoolClosed);
@@ -340,6 +434,42 @@ impl Drop for Connection {
     }
 }
 
+/// A handle to a pinned connection - the connection itself can be retrieved or returned to the
+/// normal pool via this handle.
+#[derive(Debug)]
+pub(crate) struct PinnedConnectionHandle {
+    id: u32,
+    receiver: Arc<Mutex<mpsc::Receiver<Connection>>>,
+}
+
+impl PinnedConnectionHandle {
+    /// Make a new `PinnedConnectionHandle` that refers to the same connection as this one.
+    /// Use with care and only when "lending" a handle in a way that can't be expressed as a
+    /// normal borrow.
+    pub(crate) fn replicate(&self) -> Self {
+        Self {
+            id: self.id,
+            receiver: self.receiver.clone(),
+        }
+    }
+
+    /// Retrieve the pinned connection, blocking until it's available for use.  Will fail if the
+    /// connection has been unpinned.
+    pub(crate) async fn take_connection(&self) -> Result<Connection> {
+        let mut receiver = self.receiver.lock().await;
+        receiver.recv().await.ok_or_else(|| {
+            Error::internal(format!(
+                "cannot take connection after unpin (id={})",
+                self.id
+            ))
+        })
+    }
+
+    pub(crate) fn id(&self) -> u32 {
+        self.id
+    }
+}
+
 #[derive(Debug, Clone)]
 pub(crate) enum ConnectionGeneration {
     Normal(u32),
