@@ -1,6 +1,6 @@
 use std::time::Duration;
 
-use crate::{client::options::ServerAddress, is_master::IsMasterReply, sdam::ServerType};
+use crate::{client::options::ServerAddress, hello::HelloReply, sdam::ServerType};
 
 /// Contains information about a given server in a format digestible by a connection.
 #[derive(Debug, Default, Clone)]
@@ -32,24 +32,27 @@ pub(crate) struct StreamDescription {
     /// can be included in a write batch.  If more than this number of writes are included, the
     /// server cannot guarantee space in the response document to reply to the batch.
     pub(crate) max_write_batch_size: i64,
+
+    /// Whether the server associated with this connection supports the `hello` command.
+    pub(crate) hello_ok: bool,
 }
 
 impl StreamDescription {
-    /// Constructs a new StreamDescription from an IsMasterReply.
-    pub(crate) fn from_is_master(reply: IsMasterReply) -> Self {
+    /// Constructs a new StreamDescription from a `HelloReply`.
+    pub(crate) fn from_hello_reply(reply: &HelloReply) -> Self {
         Self {
-            server_address: reply.server_address,
+            server_address: reply.server_address.clone(),
             initial_server_type: reply.command_response.server_type(),
             max_wire_version: reply.command_response.max_wire_version,
             min_wire_version: reply.command_response.min_wire_version,
-            sasl_supported_mechs: reply.command_response.sasl_supported_mechs,
-            // TODO RUST-204: Add "saslSupportedMechs" if applicable.
+            sasl_supported_mechs: reply.command_response.sasl_supported_mechs.clone(),
             logical_session_timeout: reply
                 .command_response
                 .logical_session_timeout_minutes
                 .map(|mins| Duration::from_secs(mins as u64 * 60)),
             max_bson_object_size: reply.command_response.max_bson_object_size,
             max_write_batch_size: reply.command_response.max_write_batch_size,
+            hello_ok: reply.command_response.hello_ok.unwrap_or(false),
         }
     }
 
@@ -78,6 +81,7 @@ impl StreamDescription {
             logical_session_timeout: Some(Duration::from_secs(30 * 60)),
             max_bson_object_size: 16 * 1024 * 1024,
             max_write_batch_size: 100_000,
+            hello_ok: false,
         }
     }
 }
