@@ -1,7 +1,5 @@
 mod header;
 mod message;
-#[cfg(test)]
-mod test;
 mod util;
 
 pub(crate) use self::{message::Message, util::next_request_id};
