@@ -10,14 +10,15 @@ use crate::{
     bson::{doc, Bson, Document},
     client::auth::{ClientFirst, FirstRound},
     cmap::{options::ConnectionPoolOptions, Command, Connection, StreamDescription},
+    compression::Compressor,
     error::{ErrorKind, Result},
     event::sdam::SdamEventHandler,
-    is_master::{is_master_command, run_is_master, IsMasterReply},
+    hello::{hello_command, run_hello, HelloReply},
     options::{AuthMechanism, ClientOptions, Credential, DriverInfo, ServerApi},
     sdam::Topology,
 };
 
-#[cfg(feature = "tokio-runtime")]
+#[cfg(all(feature = "tokio-runtime", not(feature = "tokio-sync")))]
 const RUNTIME_NAME: &str = "tokio";
 
 #[cfg(all(feature = "async-std-runtime", not(feature = "sync")))]
@@ -26,12 +27,15 @@ const RUNTIME_NAME: &str = "async-std";
 #[cfg(feature = "sync")]
 const RUNTIME_NAME: &str = "sync (with async-std)";
 
+#[cfg(feature = "tokio-sync")]
+const RUNTIME_NAME: &str = "sync (with tokio)";
+
 #[derive(Clone, Debug)]
 struct ClientMetadata {
     application: Option<AppMetadata>,
     driver: DriverMetadata,
     os: OsMetadata,
-    platform: Option<String>,
+    platform: String,
 }
 
 #[derive(Clone, Debug)]
@@ -70,10 +74,7 @@ impl From<ClientMetadata> for Bson {
         );
 
         metadata_doc.insert("os", metadata.os);
-
-        if let Some(platform) = metadata.platform {
-            metadata_doc.insert("platform", platform);
-        }
+        metadata_doc.insert("platform", metadata.platform);
 
         Bson::Document(metadata_doc)
     }
@@ -100,7 +101,7 @@ impl From<OsMetadata> for Bson {
 lazy_static! {
     /// Contains the basic handshake information that can be statically determined. This document
     /// (potentially with additional fields added) can be cloned and put in the `client` field of
-    /// the `isMaster` command.
+    /// the `hello` or legacy hello command.
     static ref BASE_CLIENT_METADATA: ClientMetadata = {
         let mut metadata = ClientMetadata {
             application: None,
@@ -114,7 +115,7 @@ lazy_static! {
                 name: None,
                 version: None,
             },
-            platform: None,
+            platform: format!("{} with {}", rustc_version_runtime::version_meta().short_version_string, RUNTIME_NAME),
         };
 
         let info = os_info::get();
@@ -127,11 +128,6 @@ lazy_static! {
             }
         }
 
-        if let Some((version, channel, date)) = version_check::triple() {
-            metadata.platform =
-                Some(format!("rustc {} {} ({}) with {}", version, channel, date, RUNTIME_NAME));
-        }
-
         metadata
     };
 }
@@ -139,10 +135,15 @@ lazy_static! {
 /// Contains the logic needed to handshake a connection.
 #[derive(Clone, Debug)]
 pub(crate) struct Handshaker {
-    /// The `isMaster` command to send when handshaking. This will always be identical
+    /// The hello or legacy hello command to send when handshaking. This will always be identical
     /// given the same pool options, so it can be created at the time the Handshaker is created.
     command: Command,
+
     credential: Option<Credential>,
+
+    // This field is not read without a compression feature flag turned on.
+    #[allow(dead_code)]
+    compressors: Option<Vec<Compressor>>,
 }
 
 impl Handshaker {
@@ -151,8 +152,12 @@ impl Handshaker {
         let mut metadata = BASE_CLIENT_METADATA.clone();
         let mut credential = None;
 
-        let mut command =
-            is_master_command(options.as_ref().and_then(|opts| opts.server_api.as_ref()));
+        let mut compressors = None;
+
+        let mut command = hello_command(
+            options.as_ref().and_then(|opts| opts.server_api.as_ref()),
+            None,
+        );
 
         if let Some(options) = options {
             if let Some(app_name) = options.app_name {
@@ -168,11 +173,9 @@ impl Handshaker {
                     metadata.driver.version.push_str(version);
                 }
 
-                if let Some(ref mut platform) = metadata.platform {
-                    if let Some(ref driver_info_platform) = driver_info.platform {
-                        platform.push('|');
-                        platform.push_str(driver_info_platform);
-                    }
+                if let Some(ref driver_info_platform) = driver_info.platform {
+                    metadata.platform.push('|');
+                    metadata.platform.push_str(driver_info_platform);
                 }
             }
 
@@ -185,6 +188,19 @@ impl Handshaker {
             if options.load_balanced {
                 command.body.insert("loadBalanced", true);
             }
+
+            // Add compressors to handshake.
+            // See https://github.com/mongodb/specifications/blob/master/source/compression/OP_COMPRESSED.rst
+            if let Some(ref compressors) = options.compressors {
+                command.body.insert(
+                    "compression",
+                    compressors
+                        .iter()
+                        .map(|x| x.name())
+                        .collect::<Vec<&'static str>>(),
+                );
+            }
+            compressors = options.compressors;
         }
 
         command.body.insert("client", metadata);
@@ -192,6 +208,7 @@ impl Handshaker {
         Self {
             command,
             credential,
+            compressors,
         }
     }
 
@@ -206,9 +223,10 @@ impl Handshaker {
 
         let client_first = set_speculative_auth_info(&mut command.body, self.credential.as_ref())?;
 
-        let mut is_master_reply = run_is_master(conn, command, topology, handler).await?;
+        let mut hello_reply = run_hello(conn, command, topology, handler).await?;
+
         if self.command.body.contains_key("loadBalanced")
-            && is_master_reply.command_response.service_id.is_none()
+            && hello_reply.command_response.service_id.is_none()
         {
             return Err(ErrorKind::IncompatibleServer {
                 message: "Driver attempted to initialize in load balancing mode, but the server \
@@ -217,20 +235,41 @@ impl Handshaker {
             }
             .into());
         }
-        conn.stream_description = Some(StreamDescription::from_is_master(is_master_reply.clone()));
+        conn.stream_description = Some(StreamDescription::from_hello_reply(&hello_reply));
 
         // Record the client's message and the server's response from speculative authentication if
         // the server did send a response.
         let first_round = client_first.and_then(|client_first| {
-            is_master_reply
+            hello_reply
                 .command_response
                 .speculative_authenticate
                 .take()
                 .map(|server_first| client_first.into_first_round(server_first))
         });
 
+        // Check that master reply has a compressor list and unpack it
+        if let (Some(server_compressors), Some(client_compressors)) = (
+            hello_reply.command_response.compressors.as_ref(),
+            self.compressors.as_ref(),
+        ) {
+            // Use the Client's first compressor choice that the server supports (comparing only on
+            // enum variant)
+            if let Some(compressor) = client_compressors
+                .iter()
+                .find(|c| server_compressors.iter().any(|x| c.name() == x))
+            {
+                // Without a feature flag turned on, the Compressor enum is empty which causes an
+                // unreachable code warning.
+                #[allow(unreachable_code)]
+                // zlib compression level is already set
+                {
+                    conn.compressor = Some(compressor.clone());
+                }
+            }
+        }
+
         Ok(HandshakeResult {
-            is_master_reply,
+            hello_reply,
             first_round,
         })
     }
@@ -243,7 +282,7 @@ impl Handshaker {
 #[derive(Debug)]
 pub(crate) struct HandshakeResult {
     /// The response from the server.
-    pub(crate) is_master_reply: IsMasterReply,
+    pub(crate) hello_reply: HelloReply,
 
     /// The first round of speculative authentication, if applicable.
     pub(crate) first_round: Option<FirstRound>,
@@ -253,6 +292,7 @@ pub(crate) struct HandshakeResult {
 pub(crate) struct HandshakerOptions {
     app_name: Option<String>,
     credential: Option<Credential>,
+    compressors: Option<Vec<Compressor>>,
     driver_info: Option<DriverInfo>,
     server_api: Option<ServerApi>,
     load_balanced: bool,
@@ -262,6 +302,7 @@ impl From<ConnectionPoolOptions> for HandshakerOptions {
     fn from(options: ConnectionPoolOptions) -> Self {
         Self {
             app_name: options.app_name,
+            compressors: options.compressors,
             credential: options.credential,
             driver_info: options.driver_info,
             server_api: options.server_api,
@@ -274,6 +315,7 @@ impl From<ClientOptions> for HandshakerOptions {
     fn from(options: ClientOptions) -> Self {
         Self {
             app_name: options.app_name,
+            compressors: options.compressors,
             credential: options.credential,
             driver_info: options.driver_info,
             server_api: options.server_api,
