@@ -1,4 +1,6 @@
 use tokio::sync::mpsc;
+#[cfg(test)]
+use tokio::sync::oneshot;
 
 use super::Connection;
 use crate::{bson::oid::ObjectId, error::Error, runtime::AcknowledgedMessage};
@@ -25,7 +27,7 @@ impl PoolManager {
         if self
             .sender
             .send(PoolManagementRequest::Clear {
-                completion_handler: message,
+                _completion_handler: message,
                 cause,
                 service_id,
             })
@@ -41,7 +43,7 @@ impl PoolManager {
         if self
             .sender
             .send(PoolManagementRequest::MarkAsReady {
-                completion_handler: message,
+                _completion_handler: message,
             })
             .is_ok()
         {
@@ -52,7 +54,10 @@ impl PoolManager {
     /// Check in the given connection to the pool.
     /// This returns an error containing the connection if the pool has been dropped already.
     pub(crate) fn check_in(&self, connection: Connection) -> std::result::Result<(), Connection> {
-        if let Err(request) = self.sender.send(PoolManagementRequest::CheckIn(connection)) {
+        if let Err(request) = self
+            .sender
+            .send(PoolManagementRequest::CheckIn(Box::new(connection)))
+        {
             let conn = request.0.unwrap_check_in();
             return Err(conn);
         }
@@ -72,6 +77,14 @@ impl PoolManager {
             .sender
             .send(PoolManagementRequest::HandleConnectionSucceeded(conn));
     }
+
+    /// Create a synchronization point for the pool's worker.
+    #[cfg(test)]
+    pub(super) fn sync_worker(&self) -> oneshot::Receiver<()> {
+        let (tx, rx) = oneshot::channel();
+        let _ = self.sender.send(PoolManagementRequest::Sync(tx));
+        rx
+    }
 }
 
 #[derive(Debug)]
@@ -89,18 +102,18 @@ impl ManagementRequestReceiver {
 pub(super) enum PoolManagementRequest {
     /// Clear the pool, transitioning it to Paused.
     Clear {
-        completion_handler: AcknowledgedMessage<()>,
+        _completion_handler: AcknowledgedMessage<()>,
         cause: Error,
         service_id: Option<ObjectId>,
     },
 
     /// Mark the pool as Ready, allowing connections to be created and checked out.
     MarkAsReady {
-        completion_handler: AcknowledgedMessage<()>,
+        _completion_handler: AcknowledgedMessage<()>,
     },
 
     /// Check in the given connection.
-    CheckIn(Connection),
+    CheckIn(Box<Connection>),
 
     /// Update the pool based on the given establishment error.
     HandleConnectionFailed,
@@ -108,12 +121,16 @@ pub(super) enum PoolManagementRequest {
     /// Update the pool after a successful connection, optionally populating the pool
     /// with the successful connection.
     HandleConnectionSucceeded(ConnectionSucceeded),
+
+    /// Synchronize the worker queue state with an external caller, i.e. a test.
+    #[cfg(test)]
+    Sync(oneshot::Sender<()>),
 }
 
 impl PoolManagementRequest {
     fn unwrap_check_in(self) -> Connection {
         match self {
-            PoolManagementRequest::CheckIn(conn) => conn,
+            PoolManagementRequest::CheckIn(conn) => *conn,
             _ => panic!("tried to unwrap checkin but got {:?}", self),
         }
     }
@@ -121,7 +138,7 @@ impl PoolManagementRequest {
 
 #[derive(Debug)]
 pub(super) enum ConnectionSucceeded {
-    ForPool(Connection),
+    ForPool(Box<Connection>),
     Used { service_id: Option<ObjectId> },
 }
 
