@@ -6,11 +6,18 @@ use futures_util::stream::TryStreamExt;
 
 use crate::{
     bson::{Bson, Document},
+    change_stream::{
+        event::ChangeStreamEvent,
+        options::ChangeStreamOptions,
+        session::SessionChangeStream,
+        ChangeStream,
+    },
     client::session::TransactionState,
+    cmap::conn::PinnedConnectionHandle,
     concern::{ReadConcern, WriteConcern},
     cursor::Cursor,
     error::{Error, ErrorKind, Result},
-    operation::{Aggregate, Create, DropDatabase, ListCollections, RunCommand},
+    operation::{Aggregate, AggregateTarget, Create, DropDatabase, ListCollections, RunCommand},
     options::{
         AggregateOptions,
         CollectionOptions,
@@ -39,7 +46,7 @@ use crate::{
 ///
 /// ```rust
 /// 
-/// # #[cfg(not(feature = "sync"))]
+/// # #[cfg(all(not(feature = "sync"), not(feature = "tokio-sync")))]
 /// # use mongodb::{bson::Document, Client, error::Result};
 /// # #[cfg(feature = "async-std-runtime")]
 /// # use async_std::task;
@@ -47,7 +54,7 @@ use crate::{
 /// # use tokio::task;
 /// #
 /// #
-/// # #[cfg(not(feature = "sync"))]
+/// # #[cfg(all(not(feature = "sync"), not(feature = "tokio-sync")))]
 /// # async fn start_workers() -> Result<()> {
 /// # let client = Client::with_uri_str("mongodb://example.com").await?;
 /// let db = client.database("items");
@@ -205,7 +212,6 @@ impl Database {
         self.client()
             .execute_cursor_operation(list_collections)
             .await
-            .map(|(spec, session)| Cursor::new(self.client().clone(), spec, session))
     }
 
     /// Gets information about each of the collections in the database using the provided
@@ -224,9 +230,8 @@ impl Database {
             options.into(),
         );
         self.client()
-            .execute_operation(list_collections, session)
+            .execute_session_cursor_operation(list_collections, session)
             .await
-            .map(|spec| SessionCursor::new(self.client().clone(), spec))
     }
 
     async fn list_collection_names_common(
@@ -258,8 +263,7 @@ impl Database {
         let cursor: Cursor<Document> = self
             .client()
             .execute_cursor_operation(list_collections)
-            .await
-            .map(|(spec, session)| Cursor::new(self.client().clone(), spec, session))?;
+            .await?;
 
         self.list_collection_names_common(cursor).await
     }
@@ -274,9 +278,8 @@ impl Database {
             ListCollections::new(self.name().to_string(), filter.into(), true, None);
         let mut cursor: SessionCursor<Document> = self
             .client()
-            .execute_operation(list_collections, Some(&mut *session))
-            .await
-            .map(|spec| SessionCursor::new(self.client().clone(), spec))?;
+            .execute_session_cursor_operation(list_collections, &mut *session)
+            .await?;
 
         self.list_collection_names_common(cursor.stream(session))
             .await
@@ -327,13 +330,19 @@ impl Database {
         self.create_collection_common(name, options, session).await
     }
 
-    async fn run_command_common(
+    pub(crate) async fn run_command_common(
         &self,
         command: Document,
         selection_criteria: impl Into<Option<SelectionCriteria>>,
         session: impl Into<Option<&mut ClientSession>>,
+        pinned_connection: Option<&PinnedConnectionHandle>,
     ) -> Result<Document> {
-        let operation = RunCommand::new(self.name().into(), command, selection_criteria.into())?;
+        let operation = RunCommand::new(
+            self.name().into(),
+            command,
+            selection_criteria.into(),
+            pinned_connection,
+        )?;
         self.client().execute_operation(operation, session).await
     }
 
@@ -347,7 +356,7 @@ impl Database {
         command: Document,
         selection_criteria: impl Into<Option<SelectionCriteria>>,
     ) -> Result<Document> {
-        self.run_command_common(command, selection_criteria, None)
+        self.run_command_common(command, selection_criteria, None, None)
             .await
     }
 
@@ -388,7 +397,7 @@ impl Database {
             }
             _ => {}
         }
-        self.run_command_common(command, selection_criteria, session)
+        self.run_command_common(command, selection_criteria, session, None)
             .await
     }
 
@@ -410,10 +419,7 @@ impl Database {
 
         let aggregate = Aggregate::new(self.name().to_string(), pipeline, options);
         let client = self.client();
-        client
-            .execute_cursor_operation(aggregate)
-            .await
-            .map(|(spec, session)| Cursor::new(client.clone(), spec, session))
+        client.execute_cursor_operation(aggregate).await
     }
 
     /// Runs an aggregation operation with the provided `ClientSession`.
@@ -436,8 +442,54 @@ impl Database {
         let aggregate = Aggregate::new(self.name().to_string(), pipeline, options);
         let client = self.client();
         client
-            .execute_operation(aggregate, session)
+            .execute_session_cursor_operation(aggregate, session)
+            .await
+    }
+
+    /// Starts a new [`ChangeStream`](change_stream/struct.ChangeStream.html) that receives events
+    /// for all changes in this database. The stream does not observe changes from system
+    /// collections and cannot be started on "config", "local" or "admin" databases.
+    ///
+    /// See the documentation [here](https://docs.mongodb.com/manual/changeStreams/) on change
+    /// streams.
+    ///
+    /// Change streams require either a "majority" read concern or no read
+    /// concern. Anything else will cause a server error.
+    ///
+    /// Note that using a `$project` stage to remove any of the `_id`, `operationType` or `ns`
+    /// fields will cause an error. The driver requires these fields to support resumability. For
+    /// more information on resumability, see the documentation for
+    /// [`ChangeStream`](change_stream/struct.ChangeStream.html)
+    ///
+    /// If the pipeline alters the structure of the returned events, the parsed type will need to be
+    /// changed via [`ChangeStream::with_type`].
+    pub async fn watch(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+    ) -> Result<ChangeStream<ChangeStreamEvent<Document>>> {
+        let mut options = options.into();
+        resolve_options!(self, options, [read_concern, selection_criteria]);
+        let target = AggregateTarget::Database(self.name().to_string());
+        self.client()
+            .execute_watch(pipeline, options, target, None)
+            .await
+    }
+
+    /// Starts a new [`SessionChangeStream`] that receives events for all changes in this database
+    /// using the provided [`ClientSession`].  See [`Database::watch`] for more information.
+    pub async fn watch_with_session(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+        session: &mut ClientSession,
+    ) -> Result<SessionChangeStream<ChangeStreamEvent<Document>>> {
+        let mut options = options.into();
+        resolve_read_concern_with_session!(self, options, Some(&mut *session))?;
+        resolve_selection_criteria_with_session!(self, options, Some(&mut *session))?;
+        let target = AggregateTarget::Database(self.name().to_string());
+        self.client()
+            .execute_watch_with_session(pipeline, options, target, None, session)
             .await
-            .map(|spec| SessionCursor::new(client.clone(), spec))
     }
 }
