@@ -11,9 +11,8 @@ use crate::{
     Namespace,
 };
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_many() {
+#[test]
+fn build_many() {
     let ns = Namespace {
         db: "test_db".to_string(),
         coll: "test_coll".to_string(),
@@ -54,9 +53,8 @@ async fn build_many() {
     assert_eq!(cmd.body, expected_body);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_one() {
+#[test]
+fn build_one() {
     let ns = Namespace {
         db: "test_db".to_string(),
         coll: "test_coll".to_string(),
@@ -97,9 +95,8 @@ async fn build_one() {
     assert_eq!(cmd.body, expected_body);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_success() {
+#[test]
+fn handle_success() {
     let op = Delete::empty();
 
     let delete_result = handle_response_test(
@@ -113,9 +110,8 @@ async fn handle_success() {
     assert_eq!(delete_result.deleted_count, 3);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_invalid_response() {
+#[test]
+fn handle_invalid_response() {
     let op = Delete::empty();
     handle_response_test(
         &op,
@@ -127,9 +123,8 @@ async fn handle_invalid_response() {
     .expect_err("should fail");
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_write_failure() {
+#[test]
+fn handle_write_failure() {
     let op = Delete::empty();
 
     let write_error_response = doc! {
@@ -158,9 +153,8 @@ async fn handle_write_failure() {
     };
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_write_concern_failure() {
+#[test]
+fn handle_write_concern_failure() {
     let op = Delete::empty();
 
     let wc_error_response = doc! {
@@ -193,6 +187,7 @@ async fn handle_write_concern_failure() {
                     "wtimeout": 0,
                     "provenance": "clientSupplied"
                 } }),
+                labels: vec![],
             };
             assert_eq!(wc_error, &expected_wc_err);
         }
