@@ -9,7 +9,7 @@ use crate::{
         Find,
         Operation,
     },
-    options::{CursorType, FindOptions, Hint, ReadConcern, ReadConcernLevel, ServerAddress},
+    options::{CursorType, FindOptions, Hint, ReadConcern, ReadConcernLevel},
     Namespace,
 };
 
@@ -19,7 +19,7 @@ fn build_test(
     options: Option<FindOptions>,
     mut expected_body: Document,
 ) {
-    let mut find = Find::<Document>::new(ns.clone(), filter, options);
+    let mut find = Find::new(ns.clone(), filter, options);
 
     let cmd = find.build(&StreamDescription::new_testing()).unwrap();
 
@@ -35,9 +35,8 @@ fn build_test(
     assert_eq!(cmd_doc, expected_body);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build() {
+#[test]
+fn build() {
     let ns = Namespace {
         db: "test_db".to_string(),
         coll: "test_coll".to_string(),
@@ -75,9 +74,8 @@ async fn build() {
     build_test(ns, Some(filter), Some(options), expected_body);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_cursor_type() {
+#[test]
+fn build_cursor_type() {
     let ns = Namespace {
         db: "test_db".to_string(),
         coll: "test_coll".to_string(),
@@ -125,9 +123,8 @@ async fn build_cursor_type() {
     build_test(ns, None, Some(tailable_await_options), tailable_await_body);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_max_await_time() {
+#[test]
+fn build_max_await_time() {
     let ns = Namespace {
         db: "test_db".to_string(),
         coll: "test_coll".to_string(),
@@ -147,9 +144,8 @@ async fn build_max_await_time() {
     build_test(ns, None, Some(options), body);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_limit() {
+#[test]
+fn build_limit() {
     let ns = Namespace {
         db: "test_db".to_string(),
         coll: "test_coll".to_string(),
@@ -177,9 +173,8 @@ async fn build_limit() {
     build_test(ns, None, Some(negative_options), negative_body);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_batch_size() {
+#[test]
+fn build_batch_size() {
     let options = FindOptions::builder().batch_size(1).build();
     let body = doc! {
         "find": "",
@@ -191,81 +186,24 @@ async fn build_batch_size() {
     let options = FindOptions::builder()
         .batch_size((std::i32::MAX as u32) + 1)
         .build();
-    let mut op = Find::<Document>::new(Namespace::empty(), None, Some(options));
+    let mut op = Find::new(Namespace::empty(), None, Some(options));
     assert!(op.build(&StreamDescription::new_testing()).is_err())
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn op_selection_criteria() {
+#[test]
+fn op_selection_criteria() {
     test::op_selection_criteria(|selection_criteria| {
         let options = FindOptions {
             selection_criteria,
             ..Default::default()
         };
-        Find::<Document>::new(Namespace::empty(), None, Some(options))
+        Find::new(Namespace::empty(), None, Some(options))
     });
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_success() {
-    let ns = Namespace {
-        db: "test_db".to_string(),
-        coll: "test_coll".to_string(),
-    };
-
-    let address = ServerAddress::Tcp {
-        host: "localhost".to_string(),
-        port: None,
-    };
-
-    let find = Find::<Document>::empty();
-
-    let first_batch = vec![doc! {"_id": 1}, doc! {"_id": 2}];
-
-    let response = doc! {
-        "cursor": {
-            "id": 123,
-            "ns": format!("{}.{}", ns.db, ns.coll),
-            "firstBatch": bson_util::to_bson_array(&first_batch),
-        },
-        "ok": 1.0
-    };
-
-    let cursor_spec = handle_response_test(&find, response.clone()).unwrap();
-    assert_eq!(cursor_spec.address(), &address);
-    assert_eq!(cursor_spec.id(), 123);
-    assert_eq!(cursor_spec.batch_size(), None);
-    assert_eq!(
-        cursor_spec
-            .initial_buffer
-            .into_iter()
-            .collect::<Vec<Document>>(),
-        first_batch
-    );
-
-    let find = Find::new(
-        ns,
-        None,
-        Some(FindOptions::builder().batch_size(123).build()),
-    );
-    let cursor_spec = handle_response_test(&find, response).unwrap();
-    assert_eq!(cursor_spec.address(), &address);
-    assert_eq!(cursor_spec.id(), 123);
-    assert_eq!(cursor_spec.batch_size(), Some(123));
-    assert_eq!(
-        cursor_spec
-            .initial_buffer
-            .into_iter()
-            .collect::<Vec<Document>>(),
-        first_batch
-    );
-}
-
 fn verify_max_await_time(max_await_time: Option<Duration>, cursor_type: Option<CursorType>) {
     let ns = Namespace::empty();
-    let find = Find::<Document>::new(
+    let find = Find::new(
         ns,
         None,
         Some(FindOptions {
@@ -290,9 +228,8 @@ fn verify_max_await_time(max_await_time: Option<Duration>, cursor_type: Option<C
     assert_eq!(spec.max_time(), max_await_time);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_max_await_time() {
+#[test]
+fn handle_max_await_time() {
     verify_max_await_time(None, None);
     verify_max_await_time(Some(Duration::from_millis(5)), None);
     verify_max_await_time(
@@ -306,10 +243,9 @@ async fn handle_max_await_time() {
     );
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_invalid_response() {
-    let find = Find::<Document>::empty();
+#[test]
+fn handle_invalid_response() {
+    let find = Find::empty();
 
     let garbled = doc! { "asdfasf": "ASdfasdf" };
     handle_response_test(&find, garbled).unwrap_err();
