@@ -1,14 +1,15 @@
 #[cfg(test)]
 mod test;
 
-use std::{collections::VecDeque, marker::PhantomData, time::Duration};
+use std::{collections::VecDeque, time::Duration};
 
-use bson::Document;
-use serde::{de::DeserializeOwned, Deserialize};
+use bson::{Document, RawDocumentBuf};
+use serde::Deserialize;
 
 use crate::{
     bson::doc,
-    cmap::{conn::PinnedConnectionHandle, Command, StreamDescription},
+    change_stream::event::ResumeToken,
+    cmap::{conn::PinnedConnectionHandle, Command, RawCommandResponse, StreamDescription},
     cursor::CursorInformation,
     error::{ErrorKind, Result},
     operation::Operation,
@@ -17,20 +18,17 @@ use crate::{
     Namespace,
 };
 
-use super::CommandResponse;
-
 #[derive(Debug)]
-pub(crate) struct GetMore<'conn, T> {
+pub(crate) struct GetMore<'conn> {
     ns: Namespace,
     cursor_id: i64,
     selection_criteria: SelectionCriteria,
     batch_size: Option<u32>,
     max_time: Option<Duration>,
     pinned_connection: Option<&'conn PinnedConnectionHandle>,
-    _phantom: PhantomData<T>,
 }
 
-impl<'conn, T> GetMore<'conn, T> {
+impl<'conn> GetMore<'conn> {
     pub(crate) fn new(
         info: CursorInformation,
         pinned: Option<&'conn PinnedConnectionHandle>,
@@ -42,15 +40,13 @@ impl<'conn, T> GetMore<'conn, T> {
             batch_size: info.batch_size,
             max_time: info.max_time,
             pinned_connection: pinned,
-            _phantom: Default::default(),
         }
     }
 }
 
-impl<'conn, T: DeserializeOwned> Operation for GetMore<'conn, T> {
-    type O = GetMoreResult<T>;
+impl<'conn> Operation for GetMore<'conn> {
+    type O = GetMoreResult;
     type Command = Document;
-    type Response = CommandResponse<GetMoreResponseBody<T>>;
 
     const NAME: &'static str = "getMore";
 
@@ -84,12 +80,15 @@ impl<'conn, T: DeserializeOwned> Operation for GetMore<'conn, T> {
 
     fn handle_response(
         &self,
-        response: GetMoreResponseBody<T>,
+        response: RawCommandResponse,
         _description: &StreamDescription,
     ) -> Result<Self::O> {
+        let response: GetMoreResponseBody = response.body()?;
+
         Ok(GetMoreResult {
             batch: response.cursor.next_batch,
             exhausted: response.cursor.id == 0,
+            post_batch_resume_token: ResumeToken::from_raw(response.cursor.post_batch_resume_token),
         })
     }
 
@@ -103,13 +102,14 @@ impl<'conn, T: DeserializeOwned> Operation for GetMore<'conn, T> {
 }
 
 #[derive(Debug, Deserialize)]
-pub(crate) struct GetMoreResponseBody<T> {
-    cursor: NextBatchBody<T>,
+pub(crate) struct GetMoreResponseBody {
+    cursor: NextBatchBody,
 }
 
 #[derive(Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
-struct NextBatchBody<T> {
+struct NextBatchBody {
     id: i64,
-    next_batch: VecDeque<T>,
+    next_batch: VecDeque<RawDocumentBuf>,
+    post_batch_resume_token: Option<RawDocumentBuf>,
 }
