@@ -1,6 +1,7 @@
+use bson::RawDocumentBuf;
+
 use crate::{
-    bson::{doc, Bson, Document},
-    bson_util,
+    bson::{doc, Bson},
     cmap::StreamDescription,
     error::ErrorKind,
     operation::{test::handle_response_test, ListDatabases, Operation},
@@ -8,9 +9,8 @@ use crate::{
     selection_criteria::ReadPreference,
 };
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build() {
+#[test]
+fn build() {
     let mut list_databases_op = ListDatabases::empty();
     let list_databases_command = list_databases_op
         .build(&StreamDescription::new_testing())
@@ -25,9 +25,8 @@ async fn build() {
     assert_eq!(list_databases_command.target_db, "admin");
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_with_name_only() {
+#[test]
+fn build_with_name_only() {
     let name_only = true;
 
     let mut list_databases_op = ListDatabases::new(None, name_only, None);
@@ -45,9 +44,8 @@ async fn build_with_name_only() {
     assert_eq!(list_databases_command.target_db, "admin");
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_with_filter() {
+#[test]
+fn build_with_filter() {
     let filter = doc! {"something" : "something else"};
 
     let mut list_databases_op = ListDatabases::new(Some(filter.clone()), false, None);
@@ -65,9 +63,8 @@ async fn build_with_filter() {
     assert_eq!(list_databases_command.target_db, "admin");
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn build_with_options() {
+#[test]
+fn build_with_options() {
     let options = ListDatabasesOptions::builder()
         .authorized_databases(true)
         .build();
@@ -87,12 +84,11 @@ async fn build_with_options() {
     assert_eq!(list_databases_command.target_db, "admin");
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_success() {
+#[test]
+fn handle_success() {
     let total_size = 251658240;
 
-    let databases: Vec<Document> = vec![
+    let databases = vec![
         doc! {
            "name" : "admin",
            "sizeOnDisk" : 83886080,
@@ -110,22 +106,26 @@ async fn handle_success() {
         },
     ];
 
+    let raw_databases = databases
+        .iter()
+        .map(|d| RawDocumentBuf::from_document(d).unwrap())
+        .collect::<Vec<_>>();
+
     let actual_values = handle_response_test(
         &ListDatabases::empty(),
         doc! {
-           "databases" : bson_util::to_bson_array(&databases),
+           "databases" : databases,
            "totalSize" : total_size,
            "ok" : 1
         },
     )
     .expect("should succeed");
 
-    assert_eq!(actual_values, databases);
+    assert_eq!(actual_values, raw_databases);
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn handle_response_no_databases() {
+#[test]
+fn handle_response_no_databases() {
     let result = handle_response_test(&ListDatabases::empty(), doc! { "ok": 1 });
     match result.map_err(|e| *e.kind) {
         Err(ErrorKind::InvalidResponse { .. }) => {}
@@ -133,9 +133,8 @@ async fn handle_response_no_databases() {
     }
 }
 
-#[cfg_attr(feature = "tokio-runtime", tokio::test)]
-#[cfg_attr(feature = "async-std-runtime", async_std::test)]
-async fn op_selection_criteria() {
+#[test]
+fn op_selection_criteria() {
     let list_databases_op = ListDatabases::empty();
     assert_eq!(
         *list_databases_op
