@@ -25,14 +25,14 @@ mod test;
 
 use std::{collections::VecDeque, fmt::Debug, ops::Deref};
 
-use bson::Timestamp;
+use bson::{RawBsonRef, RawDocument, RawDocumentBuf, Timestamp};
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
 
 use crate::{
     bson::{self, Bson, Document},
     bson_util,
     client::{ClusterTime, HELLO_COMMAND_NAMES, REDACTED_COMMANDS},
-    cmap::{Command, RawCommandResponse, StreamDescription},
+    cmap::{conn::PinnedConnectionHandle, Command, RawCommandResponse, StreamDescription},
     error::{
         BulkWriteError,
         BulkWriteFailure,
@@ -49,7 +49,7 @@ use crate::{
 };
 
 pub(crate) use abort_transaction::AbortTransaction;
-pub(crate) use aggregate::Aggregate;
+pub(crate) use aggregate::{Aggregate, AggregateTarget, ChangeStreamAggregate};
 pub(crate) use commit_transaction::CommitTransaction;
 pub(crate) use count::Count;
 pub(crate) use count_documents::CountDocuments;
@@ -70,6 +70,9 @@ pub(crate) use list_indexes::ListIndexes;
 pub(crate) use run_command::RunCommand;
 pub(crate) use update::Update;
 
+const SERVER_4_9_0_WIRE_VERSION: i32 = 12;
+const SERVER_4_2_0_WIRE_VERSION: i32 = 8;
+
 /// A trait modeling the behavior of a server side operation.
 pub(crate) trait Operation {
     /// The output type of this operation.
@@ -78,9 +81,6 @@ pub(crate) trait Operation {
     /// The format of the command body constructed in `build`.
     type Command: CommandBody;
 
-    /// The format of the command response from the server.
-    type Response: Response;
-
     /// The name of the server side command associated with this operation.
     const NAME: &'static str;
 
@@ -94,10 +94,16 @@ pub(crate) trait Operation {
         Ok(bson::to_vec(&cmd)?)
     }
 
+    /// Parse the response for the atClusterTime field.
+    /// Depending on the operation, this may be found in different locations.
+    fn extract_at_cluster_time(&self, _response: &RawDocument) -> Result<Option<Timestamp>> {
+        Ok(None)
+    }
+
     /// Interprets the server response to the command.
     fn handle_response(
         &self,
-        response: <Self::Response as Response>::Body,
+        response: RawCommandResponse,
         description: &StreamDescription,
     ) -> Result<Self::O>;
 
@@ -124,6 +130,11 @@ pub(crate) trait Operation {
         None
     }
 
+    /// Returns whether or not this command supports the `readConcern` field.
+    fn supports_read_concern(&self, _description: &StreamDescription) -> bool {
+        false
+    }
+
     /// Whether this operation supports sessions or not.
     fn supports_sessions(&self) -> bool {
         true
@@ -137,6 +148,10 @@ pub(crate) trait Operation {
     /// Updates this operation as needed for a retry.
     fn update_for_retry(&mut self) {}
 
+    fn pinned_connection(&self) -> Option<&PinnedConnectionHandle> {
+        None
+    }
+
     fn name(&self) -> &str {
         Self::NAME
     }
@@ -164,39 +179,11 @@ impl<T: CommandBody> Command<T> {
         let name = self.name.to_lowercase();
         REDACTED_COMMANDS.contains(name.as_str()) || self.body.should_redact()
     }
-}
 
-/// Trait modeling the behavior of a command response to a server operation.
-pub(crate) trait Response: Sized {
-    /// The command-specific portion of a command response.
-    /// This type will be passed to the associated operation's `handle_response` method.
-    type Body;
-
-    /// Deserialize a response from the given raw response.
-    fn deserialize_response(raw: &RawCommandResponse) -> Result<Self>;
-
-    /// The `ok` field of the response.
-    fn ok(&self) -> Option<&Bson>;
-
-    /// Whether the command succeeeded or not (i.e. if this response is ok: 1).
-    fn is_success(&self) -> bool {
-        match self.ok() {
-            Some(b) => bson_util::get_int(b) == Some(1),
-            None => false,
-        }
+    pub(crate) fn should_compress(&self) -> bool {
+        let name = self.name.to_lowercase();
+        !REDACTED_COMMANDS.contains(name.as_str()) && !HELLO_COMMAND_NAMES.contains(name.as_str())
     }
-
-    /// The `clusterTime` field of the response.
-    fn cluster_time(&self) -> Option<&ClusterTime>;
-
-    /// The `atClusterTime` field of the response.
-    fn at_cluster_time(&self) -> Option<Timestamp>;
-
-    /// The `recoveryToken` field of the response.
-    fn recovery_token(&self) -> Option<&Document>;
-
-    /// Convert into the body of the response.
-    fn into_body(self) -> Self::Body;
 }
 
 /// A response to a command with a body shaped deserialized to a `T`.
@@ -208,82 +195,19 @@ pub(crate) struct CommandResponse<T> {
     #[serde(rename = "$clusterTime")]
     pub(crate) cluster_time: Option<ClusterTime>,
 
-    pub(crate) at_cluster_time: Option<Timestamp>,
-
-    pub(crate) recovery_token: Option<Document>,
-
     #[serde(flatten)]
     pub(crate) body: T,
 }
 
 impl<T: DeserializeOwned> CommandResponse<T> {
+    /// Whether the command succeeeded or not (i.e. if this response is ok: 1).
     pub(crate) fn is_success(&self) -> bool {
-        <Self as Response>::is_success(self)
-    }
-}
-
-impl<T: DeserializeOwned> Response for CommandResponse<T> {
-    type Body = T;
-
-    fn deserialize_response(raw: &RawCommandResponse) -> Result<Self> {
-        raw.body()
+        bson_util::get_int(&self.ok) == Some(1)
     }
 
-    fn ok(&self) -> Option<&Bson> {
-        Some(&self.ok)
-    }
-
-    fn cluster_time(&self) -> Option<&ClusterTime> {
+    pub(crate) fn cluster_time(&self) -> Option<&ClusterTime> {
         self.cluster_time.as_ref()
     }
-
-    fn at_cluster_time(&self) -> Option<Timestamp> {
-        self.at_cluster_time
-    }
-
-    fn recovery_token(&self) -> Option<&Document> {
-        self.recovery_token.as_ref()
-    }
-
-    fn into_body(self) -> Self::Body {
-        self.body
-    }
-}
-
-/// A response to commands that return cursors.
-#[derive(Debug)]
-pub(crate) struct CursorResponse<T> {
-    response: CommandResponse<CursorBody<T>>,
-}
-
-impl<T: DeserializeOwned> Response for CursorResponse<T> {
-    type Body = CursorBody<T>;
-
-    fn deserialize_response(raw: &RawCommandResponse) -> Result<Self> {
-        Ok(Self {
-            response: raw.body()?,
-        })
-    }
-
-    fn ok(&self) -> Option<&Bson> {
-        self.response.ok()
-    }
-
-    fn cluster_time(&self) -> Option<&ClusterTime> {
-        self.response.cluster_time()
-    }
-
-    fn at_cluster_time(&self) -> Option<Timestamp> {
-        self.response.body.cursor.at_cluster_time
-    }
-
-    fn recovery_token(&self) -> Option<&Document> {
-        self.response.recovery_token()
-    }
-
-    fn into_body(self) -> Self::Body {
-        self.response.body
-    }
 }
 
 /// A response body useful for deserializing command errors.
@@ -399,24 +323,60 @@ impl<T> Deref for WriteResponseBody<T> {
 }
 
 #[derive(Debug, Deserialize)]
-pub(crate) struct CursorBody<T> {
-    cursor: CursorInfo<T>,
+pub(crate) struct CursorBody {
+    cursor: CursorInfo,
+}
 
-    #[serde(flatten)]
-    write_concern_info: WriteConcernOnlyBody,
+impl CursorBody {
+    fn extract_at_cluster_time(response: &RawDocument) -> Result<Option<Timestamp>> {
+        Ok(response
+            .get("cursor")?
+            .and_then(RawBsonRef::as_document)
+            .map(|d| d.get("atClusterTime"))
+            .transpose()?
+            .flatten()
+            .and_then(RawBsonRef::as_timestamp))
+    }
 }
 
 #[derive(Debug, Deserialize, Clone)]
-pub(crate) struct CursorInfo<T> {
+#[serde(rename_all = "camelCase")]
+pub(crate) struct CursorInfo {
     pub(crate) id: i64,
 
     pub(crate) ns: Namespace,
 
-    #[serde(rename = "firstBatch")]
-    pub(crate) first_batch: VecDeque<T>,
+    pub(crate) first_batch: VecDeque<RawDocumentBuf>,
 
-    #[serde(rename = "atClusterTime")]
-    pub(crate) at_cluster_time: Option<Timestamp>,
+    pub(crate) post_batch_resume_token: Option<RawDocumentBuf>,
+}
+
+/// Type used to deserialize just the first result from a cursor, if any.
+#[derive(Debug, Clone)]
+pub(crate) struct SingleCursorResult<T>(Option<T>);
+
+impl<'de, T> Deserialize<'de> for SingleCursorResult<T>
+where
+    T: Deserialize<'de>,
+{
+    fn deserialize<D>(deserializer: D) -> std::result::Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        #[derive(Deserialize)]
+        struct FullCursorBody<T> {
+            cursor: InteriorBody<T>,
+        }
+
+        #[derive(Deserialize)]
+        struct InteriorBody<T> {
+            #[serde(rename = "firstBatch")]
+            first_batch: Vec<T>,
+        }
+
+        let mut full_body = FullCursorBody::deserialize(deserializer)?;
+        Ok(SingleCursorResult(full_body.cursor.first_batch.pop()))
+    }
 }
 
 #[derive(Debug, PartialEq)]
@@ -425,3 +385,17 @@ pub(crate) enum Retryability {
     Read,
     None,
 }
+
+macro_rules! remove_empty_write_concern {
+    ($opts:expr) => {
+        if let Some(ref mut options) = $opts {
+            if let Some(ref write_concern) = options.write_concern {
+                if write_concern.is_empty() {
+                    options.write_concern = None;
+                }
+            }
+        }
+    };
+}
+
+pub(crate) use remove_empty_write_concern;
