@@ -1,31 +1,35 @@
 #[cfg(test)]
 mod test;
 
-use bson::{Bson, Timestamp};
+use std::convert::TryInto;
 
-use super::Operation;
+use bson::RawBsonRef;
+
+use super::{CursorBody, Operation};
 use crate::{
     bson::Document,
-    client::{ClusterTime, SESSIONS_UNSUPPORTED_COMMANDS},
-    cmap::{Command, RawCommandResponse, StreamDescription},
+    client::SESSIONS_UNSUPPORTED_COMMANDS,
+    cmap::{conn::PinnedConnectionHandle, Command, RawCommandResponse, StreamDescription},
     error::{ErrorKind, Result},
     options::WriteConcern,
     selection_criteria::SelectionCriteria,
 };
 
 #[derive(Debug)]
-pub(crate) struct RunCommand {
+pub(crate) struct RunCommand<'conn> {
     db: String,
     command: Document,
     selection_criteria: Option<SelectionCriteria>,
     write_concern: Option<WriteConcern>,
+    pinned_connection: Option<&'conn PinnedConnectionHandle>,
 }
 
-impl RunCommand {
+impl<'conn> RunCommand<'conn> {
     pub(crate) fn new(
         db: String,
         command: Document,
         selection_criteria: Option<SelectionCriteria>,
+        pinned_connection: Option<&'conn PinnedConnectionHandle>,
     ) -> Result<Self> {
         let write_concern = command
             .get("writeConcern")
@@ -37,6 +41,7 @@ impl RunCommand {
             command,
             selection_criteria,
             write_concern,
+            pinned_connection,
         })
     }
 
@@ -45,10 +50,9 @@ impl RunCommand {
     }
 }
 
-impl Operation for RunCommand {
+impl<'conn> Operation for RunCommand<'conn> {
     type O = Document;
     type Command = Document;
-    type Response = Response;
 
     // Since we can't actually specify a string statically here, we just put a descriptive string
     // that should fail loudly if accidentally passed to the server.
@@ -68,12 +72,23 @@ impl Operation for RunCommand {
         ))
     }
 
+    fn extract_at_cluster_time(
+        &self,
+        response: &bson::RawDocument,
+    ) -> Result<Option<bson::Timestamp>> {
+        if let Some(RawBsonRef::Timestamp(ts)) = response.get("atClusterTime")? {
+            Ok(Some(ts))
+        } else {
+            CursorBody::extract_at_cluster_time(response)
+        }
+    }
+
     fn handle_response(
         &self,
-        response: Document,
+        response: RawCommandResponse,
         _description: &StreamDescription,
     ) -> Result<Self::O> {
-        Ok(response)
+        Ok(response.into_raw_document_buf().try_into()?)
     }
 
     fn selection_criteria(&self) -> Option<&SelectionCriteria> {
@@ -91,59 +106,8 @@ impl Operation for RunCommand {
             })
             .unwrap_or(false)
     }
-}
 
-#[derive(Debug)]
-pub(crate) struct Response {
-    doc: Document,
-    cluster_time: Option<ClusterTime>,
-    recovery_token: Option<Document>,
-}
-
-impl super::Response for Response {
-    type Body = Document;
-
-    fn deserialize_response(raw: &RawCommandResponse) -> Result<Self> {
-        let doc: Document = raw.body()?;
-
-        let cluster_time = doc
-            .get_document("$clusterTime")
-            .ok()
-            .and_then(|doc| bson::from_document(doc.clone()).ok());
-
-        let recovery_token = doc.get_document("recoveryToken").ok().cloned();
-
-        Ok(Self {
-            doc,
-            cluster_time,
-            recovery_token,
-        })
-    }
-
-    fn ok(&self) -> Option<&Bson> {
-        self.doc.get("ok")
-    }
-
-    fn cluster_time(&self) -> Option<&ClusterTime> {
-        self.cluster_time.as_ref()
-    }
-
-    fn at_cluster_time(&self) -> Option<Timestamp> {
-        self.doc
-            .get_timestamp("atClusterTime")
-            .or_else(|_| {
-                self.doc
-                    .get_document("cursor")
-                    .and_then(|subdoc| subdoc.get_timestamp("atClusterTime"))
-            })
-            .ok()
-    }
-
-    fn recovery_token(&self) -> Option<&Document> {
-        self.recovery_token.as_ref()
-    }
-
-    fn into_body(self) -> Self::Body {
-        self.doc
+    fn pinned_connection(&self) -> Option<&PinnedConnectionHandle> {
+        self.pinned_connection
     }
 }
