@@ -11,7 +11,7 @@ use crate::{
     client::Client,
     cmap::{conn::ConnectionGeneration, PoolGeneration},
     error::{BulkWriteFailure, CommandError, Error, ErrorKind},
-    is_master::{IsMasterCommandResponse, IsMasterReply, LastWrite},
+    hello::{HelloCommandResponse, HelloReply, LastWrite, LEGACY_HELLO_COMMAND_NAME},
     options::{ClientOptions, ReadPreference, SelectionCriteria, ServerAddress},
     sdam::{
         description::{
@@ -24,6 +24,7 @@ use crate::{
     },
     selection_criteria::TagSet,
     test::{
+        log_uncaptured,
         run_spec_test,
         Event,
         EventClient,
@@ -57,14 +58,15 @@ pub struct Phase {
 }
 
 #[derive(Debug, Deserialize)]
-pub struct Response(String, TestIsMasterCommandResponse);
+pub struct Response(String, TestHelloCommandResponse);
 
 #[derive(Debug, Clone, Default, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
-pub(crate) struct TestIsMasterCommandResponse {
+pub(crate) struct TestHelloCommandResponse {
     pub is_writable_primary: Option<bool>,
     #[serde(rename = "ismaster")]
     pub is_master: Option<bool>,
+    pub hello_ok: Option<bool>,
     pub ok: Option<f32>,
     pub hosts: Option<Vec<String>>,
     pub passives: Option<Vec<String>>,
@@ -92,9 +94,9 @@ pub(crate) struct TestIsMasterCommandResponse {
     pub service_id: Option<ObjectId>,
 }
 
-impl From<TestIsMasterCommandResponse> for IsMasterCommandResponse {
-    fn from(test: TestIsMasterCommandResponse) -> Self {
-        IsMasterCommandResponse {
+impl From<TestHelloCommandResponse> for HelloCommandResponse {
+    fn from(test: TestHelloCommandResponse) -> Self {
+        HelloCommandResponse {
             is_writable_primary: test.is_writable_primary,
             is_master: test.is_master,
             hosts: test.hosts,
@@ -120,6 +122,9 @@ impl From<TestIsMasterCommandResponse> for IsMasterCommandResponse {
             max_bson_object_size: test.max_bson_object_size.unwrap_or(1234),
             max_write_batch_size: test.max_write_batch_size.unwrap_or(1234),
             service_id: test.service_id,
+            topology_version: None,
+            compressors: None,
+            hello_ok: test.hello_ok,
         }
     }
 }
@@ -225,6 +230,7 @@ fn server_type_from_str(s: &str) -> Option<ServerType> {
         "RSArbiter" => ServerType::RsArbiter,
         "RSOther" => ServerType::RsOther,
         "RSGhost" => ServerType::RsGhost,
+        "LoadBalancer" => ServerType::LoadBalancer,
         "Unknown" | "PossiblePrimary" => ServerType::Unknown,
         _ => return None,
     };
@@ -236,15 +242,25 @@ async fn run_test(test_file: TestFile) {
     let test_description = &test_file.description;
 
     // TODO: RUST-360 unskip tests that rely on topology version
-    if test_description.contains("topologyVersion") {
-        println!("Skipping {} (RUST-360)", test_description);
-        return;
-    }
+    // TODO: RUST-358 unskip tests
+    // TODO: RUST-1081 unskip tests
+    let skip_keywords = doc! {
+        "topologyVersion": "(RUST-360)",
+        "wrong set name": "(RUST-358)",
+        "election Id": "(RUST-1081)",
+        "electionId": "(RUST-1081)",
+        "ElectionId": "(RUST-1081)"
+    };
 
-    // TODO RUST-653 unskip load balancer test
-    if test_description.contains("load balancer") {
-        println!("Skipping {} (RUST-653)", test_description);
-        return;
+    for (keyword, skip_reason) in skip_keywords {
+        if test_description.contains(keyword.as_str()) {
+            log_uncaptured(format!(
+                "Skipping {}: {}",
+                test_description,
+                skip_reason.as_str().unwrap()
+            ));
+            return;
+        }
     }
 
     let mut options = ClientOptions::parse_uri(&test_file.uri, None)
@@ -253,8 +269,9 @@ async fn run_test(test_file: TestFile) {
 
     let handler = Arc::new(EventHandler::new());
     options.sdam_event_handler = Some(handler.clone());
+    options.test_options_mut().disable_monitoring_threads = true;
 
-    let topology = Topology::new_mocked(options.clone());
+    let topology = Topology::new(options.clone()).unwrap();
     let mut servers = topology.get_servers().await;
 
     for (i, phase) in test_file.phases.into_iter().enumerate() {
@@ -267,7 +284,7 @@ async fn run_test(test_file: TestFile) {
                 )
             });
 
-            let is_master_reply = if command_response.ok != Some(1.0) {
+            let hello_reply = if command_response.ok != Some(1.0) {
                 Err(Error::from(ErrorKind::Command(CommandError {
                     code: 1234,
                     code_name: "dummy error".to_string(),
@@ -278,7 +295,7 @@ async fn run_test(test_file: TestFile) {
                     std::io::ErrorKind::BrokenPipe.into(),
                 ))))
             } else {
-                Ok(IsMasterReply {
+                Ok(HelloReply {
                     server_address: address.clone(),
                     command_response: command_response.into(),
                     round_trip_time: Duration::from_millis(1234), // Doesn't matter for tests.
@@ -288,7 +305,7 @@ async fn run_test(test_file: TestFile) {
 
             // only update server if we have strong reference to it like the monitors do
             if let Some(server) = servers.get(&address).and_then(|s| s.upgrade()) {
-                match is_master_reply {
+                match hello_reply {
                     Ok(reply) => {
                         let new_sd = ServerDescription::new(address.clone(), Some(Ok(reply)));
                         if topology.update(&server, new_sd).await {
@@ -351,7 +368,13 @@ async fn run_test(test_file: TestFile) {
             }
             Outcome::Events(EventsOutcome { events: expected }) => {
                 let actual = handler.get_all_sdam_events();
-                assert_eq!(actual.len(), expected.len());
+                assert_eq!(
+                    actual.len(),
+                    expected.len(),
+                    "event list length mismatch:\n actual: {:#?}, expected: {:#?}",
+                    actual,
+                    expected
+                );
                 for (actual, expected) in actual.iter().zip(expected.iter()) {
                     assert_eq!(
                         actual, expected,
@@ -522,6 +545,16 @@ async fn monitoring() {
     run_spec_test(&["server-discovery-and-monitoring", "monitoring"], run_test).await;
 }
 
+#[cfg_attr(feature = "tokio-runtime", tokio::test)]
+#[cfg_attr(feature = "async-std-runtime", async_std::test)]
+async fn load_balanced() {
+    run_spec_test(
+        &["server-discovery-and-monitoring", "load-balanced"],
+        run_test,
+    )
+    .await;
+}
+
 #[cfg_attr(feature = "tokio-runtime", tokio::test)]
 #[cfg_attr(feature = "async-std-runtime", async_std::test)]
 #[function_name::named]
@@ -582,6 +615,11 @@ async fn heartbeat_events() {
     )
     .await;
 
+    if client.is_load_balanced() {
+        log_uncaptured("skipping heartbeat_events tests due to load-balanced topology");
+        return;
+    }
+
     subscriber
         .wait_for_event(Duration::from_millis(500), |event| {
             matches!(event, Event::Sdam(SdamEvent::ServerHeartbeatStarted(_)))
@@ -596,13 +634,16 @@ async fn heartbeat_events() {
         .await
         .expect("should see server heartbeat succeeded event");
 
-    if !client.supports_fail_command().await {
+    if !client.supports_fail_command() {
         return;
     }
 
     let options = FailCommandOptions::builder().error_code(1234).build();
-    let failpoint =
-        FailPoint::fail_command(&["isMaster", "hello"], FailPointMode::Times(1), options);
+    let failpoint = FailPoint::fail_command(
+        &[LEGACY_HELLO_COMMAND_NAME, "hello"],
+        FailPointMode::Times(1),
+        options,
+    );
     let _fp_guard = client
         .enable_failpoint(failpoint, None)
         .await
@@ -624,7 +665,7 @@ async fn direct_connection() {
 
     let test_client = TestClient::new().await;
     if !test_client.is_replica_set() {
-        println!("Skipping due to non-replica set topology");
+        log_uncaptured("Skipping direct_connection test due to non-replica set topology");
         return;
     }
 
@@ -676,10 +717,11 @@ async fn direct_connection() {
 #[cfg_attr(feature = "async-std-runtime", async_std::test)]
 async fn pool_cleared_error_does_not_mark_unknown() {
     let address = ServerAddress::parse("a:1234").unwrap();
-    let options = ClientOptions::builder()
+    let mut options = ClientOptions::builder()
         .hosts(vec![address.clone()])
         .build();
-    let topology = Topology::new_mocked(options);
+    options.test_options_mut().disable_monitoring_threads = true;
+    let topology = Topology::new(options).unwrap();
 
     // get the one server in the topology
     let server = topology
@@ -692,9 +734,9 @@ async fn pool_cleared_error_does_not_mark_unknown() {
         .upgrade()
         .unwrap();
 
-    let heartbeat_response: IsMasterCommandResponse = bson::from_document(doc! {
+    let heartbeat_response: HelloCommandResponse = bson::from_document(doc! {
         "ok": 1,
-        "ismaster": true,
+        "isWritablePrimary": true,
         "minWireVersion": 0,
         "maxWireVersion": 6,
         "maxBsonObjectSize": 16_000,
@@ -708,7 +750,7 @@ async fn pool_cleared_error_does_not_mark_unknown() {
             &server,
             ServerDescription::new(
                 address.clone(),
-                Some(Ok(IsMasterReply {
+                Some(Ok(HelloReply {
                     server_address: address.clone(),
                     command_response: heartbeat_response,
                     round_trip_time: Duration::from_secs(1),
