@@ -11,8 +11,8 @@ use super::{
 use crate::{
     error::{Error, Result},
     options::ClientOptions,
+    runtime,
     srv::{LookupHosts, SrvResolver},
-    RUNTIME,
 };
 
 const MIN_RESCAN_SRV_INTERVAL: Duration = Duration::from_secs(60);
@@ -47,7 +47,7 @@ impl SrvPollingMonitor {
     /// set of mongos in the cluster have changed. A weak reference is used to ensure that the
     /// monitoring task doesn't keep the topology alive after the client has been dropped.
     pub(super) fn start(topology: WeakTopology) {
-        RUNTIME.execute(async move {
+        runtime::execute(async move {
             if let Some(mut monitor) = Self::new(topology) {
                 monitor.execute().await;
             }
@@ -64,7 +64,7 @@ impl SrvPollingMonitor {
         }
 
         while self.topology.is_alive() {
-            RUNTIME.delay_for(self.rescan_interval()).await;
+            runtime::delay_for(self.rescan_interval()).await;
 
             let topology = match self.topology.upgrade() {
                 Some(topology) => topology,
@@ -112,6 +112,15 @@ impl SrvPollingMonitor {
     }
 
     async fn lookup_hosts(&mut self) -> Result<LookupHosts> {
+        #[cfg(test)]
+        if let Some(mock) = self
+            .client_options
+            .test_options
+            .as_ref()
+            .and_then(|to| to.mock_lookup_hosts.as_ref())
+        {
+            return mock.clone();
+        }
         let initial_hostname = self.initial_hostname.clone();
         let resolver = self.get_or_create_srv_resolver().await?;
         resolver.get_srv_hosts(initial_hostname.as_str()).await
