@@ -6,6 +6,7 @@ use super::{LookupHosts, SrvPollingMonitor};
 use crate::{
     error::Result,
     options::{ClientOptions, ServerAddress},
+    runtime,
     sdam::Topology,
 };
 
@@ -26,22 +27,23 @@ lazy_static::lazy_static! {
 async fn run_test(new_hosts: Result<Vec<ServerAddress>>, expected_hosts: HashSet<ServerAddress>) {
     let mut options = ClientOptions::new_srv();
     options.hosts = DEFAULT_HOSTS.clone();
-    let topology = Topology::new_mocked(options);
+    options.test_options_mut().disable_monitoring_threads = true;
+    let topology = Topology::new(options).unwrap();
     let mut monitor = SrvPollingMonitor::new(topology.downgrade()).unwrap();
-
     monitor
-        .update_hosts(
-            new_hosts.map(|hosts| LookupHosts {
-                hosts: hosts.into_iter().map(Result::Ok).collect(),
-                min_ttl: Duration::from_secs(60),
-            }),
-            topology.clone(),
-        )
+        .update_hosts(new_hosts.and_then(make_lookup_hosts), topology.clone())
         .await;
 
     assert_eq!(expected_hosts, topology.servers().await);
 }
 
+fn make_lookup_hosts(hosts: Vec<ServerAddress>) -> Result<LookupHosts> {
+    Ok(LookupHosts {
+        hosts: hosts.into_iter().map(Result::Ok).collect(),
+        min_ttl: Duration::from_secs(60),
+    })
+}
+
 // If a new DNS record is returned, it should be reflected in the topology.
 #[cfg_attr(feature = "tokio-runtime", tokio::test)]
 #[cfg_attr(feature = "async-std-runtime", async_std::test)]
@@ -102,3 +104,25 @@ async fn timeout_error() {
 async fn no_results() {
     run_test(Ok(Vec::new()), DEFAULT_HOSTS.iter().cloned().collect()).await;
 }
+
+// SRV polling is not done for load-balanced clusters (as per spec at
+// https://github.com/mongodb/specifications/blob/master/source/polling-srv-records-for-mongos-discovery/tests/README.rst#test-that-srv-polling-is-not-done-for-load-balalanced-clusters).
+#[cfg_attr(feature = "tokio-runtime", tokio::test)]
+#[cfg_attr(feature = "async-std-runtime", async_std::test)]
+async fn load_balanced_no_srv_polling() {
+    let hosts = vec![localhost_test_build_10gen(27017)];
+    let mut options = ClientOptions::new_srv();
+    let rescan_interval = options.original_srv_info.as_ref().cloned().unwrap().min_ttl;
+    options.hosts = hosts.clone();
+    options.load_balanced = Some(true);
+    options.test_options_mut().mock_lookup_hosts = Some(make_lookup_hosts(vec![
+        localhost_test_build_10gen(27017),
+        localhost_test_build_10gen(27018),
+    ]));
+    let topology = Topology::new(options).unwrap();
+    runtime::delay_for(rescan_interval * 2).await;
+    assert_eq!(
+        hosts.into_iter().collect::<HashSet<_>>(),
+        topology.servers().await
+    );
+}
