@@ -36,6 +36,7 @@ use crate::{
         TopologyOpeningEvent,
     },
     options::{ClientOptions, SelectionCriteria, ServerAddress},
+    runtime,
     runtime::HttpClient,
     sdam::{
         description::{
@@ -45,7 +46,6 @@ use crate::{
         srv_polling::SrvPollingMonitor,
         TopologyMessageManager,
     },
-    RUNTIME,
 };
 
 /// A strong reference to the topology, which includes the current state as well as the client
@@ -119,7 +119,7 @@ impl Topology {
 
         // we can block in place here because we're the only ones with access to the lock, so it
         // should be acquired immediately.
-        let mut topology_state = RUNTIME.block_in_place(topology.state.write());
+        let mut topology_state = runtime::block_in_place(topology.state.write());
         for address in &options.hosts {
             topology_state.add_new_server(address.clone(), options.clone(), &topology.downgrade());
         }
@@ -348,7 +348,7 @@ impl Topology {
     ) -> bool {
         let server_type = server_description.server_type;
         // TODO RUST-580: Theoretically, `TopologyDescription::update` can return an error. However,
-        // this can only happen if we try to access a field from the isMaster response when an error
+        // this can only happen if we try to access a field from the hello response when an error
         // occurred during the check. In practice, this can't happen, because the SDAM algorithm
         // doesn't check the fields of an Unknown server, and we only return Unknown server
         // descriptions when errors occur. Once we implement logging, we can properly inform users
