@@ -22,6 +22,7 @@ pub(crate) struct ResolvedConfig {
     pub(crate) load_balanced: Option<bool>,
 }
 
+#[derive(Debug, Clone)]
 pub(crate) struct LookupHosts {
     pub(crate) hosts: Vec<Result<ServerAddress>>,
     pub(crate) min_ttl: Duration,
@@ -79,8 +80,8 @@ impl SrvResolver {
         let mut min_ttl = u32::MAX;
 
         for record in srv_lookup.as_lookup().record_iter() {
-            let srv = match record.rdata() {
-                RData::SRV(s) => s,
+            let srv = match record.data() {
+                Some(RData::SRV(s)) => s,
                 _ => continue,
             };
 
