@@ -2,9 +2,10 @@ use std::{borrow::Borrow, fmt::Debug};
 
 use serde::{de::DeserializeOwned, Serialize};
 
-use super::{ClientSession, Cursor, SessionCursor};
+use super::{ChangeStream, ClientSession, Cursor, SessionChangeStream, SessionCursor};
 use crate::{
     bson::{Bson, Document},
+    change_stream::{event::ChangeStreamEvent, options::ChangeStreamOptions},
     error::Result,
     index::IndexModel,
     options::{
@@ -39,9 +40,9 @@ use crate::{
         InsertOneResult,
         UpdateResult,
     },
+    runtime,
     Collection as AsyncCollection,
     Namespace,
-    RUNTIME,
 };
 
 /// `Collection` is the client-side abstraction of a MongoDB Collection. It can be used to
@@ -127,7 +128,7 @@ impl<T> Collection<T> {
 
     /// Drops the collection, deleting all data, users, and indexes stored in it.
     pub fn drop(&self, options: impl Into<Option<DropCollectionOptions>>) -> Result<()> {
-        RUNTIME.block_on(self.async_collection.drop(options.into()))
+        runtime::block_on(self.async_collection.drop(options.into()))
     }
 
     /// Drops the collection, deleting all data, users, and indexes stored in it using the provided
@@ -137,7 +138,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<DropCollectionOptions>>,
         session: &mut ClientSession,
     ) -> Result<()> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .drop_with_session(options.into(), &mut session.async_client_session),
         )
@@ -153,8 +154,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<AggregateOptions>>,
     ) -> Result<Cursor<Document>> {
         let pipeline: Vec<Document> = pipeline.into_iter().collect();
-        RUNTIME
-            .block_on(self.async_collection.aggregate(pipeline, options.into()))
+        runtime::block_on(self.async_collection.aggregate(pipeline, options.into()))
             .map(Cursor::new)
     }
 
@@ -169,13 +169,12 @@ impl<T> Collection<T> {
         session: &mut ClientSession,
     ) -> Result<SessionCursor<Document>> {
         let pipeline: Vec<Document> = pipeline.into_iter().collect();
-        RUNTIME
-            .block_on(self.async_collection.aggregate_with_session(
-                pipeline,
-                options.into(),
-                &mut session.async_client_session,
-            ))
-            .map(SessionCursor::new)
+        runtime::block_on(self.async_collection.aggregate_with_session(
+            pipeline,
+            options.into(),
+            &mut session.async_client_session,
+        ))
+        .map(SessionCursor::new)
     }
 
     /// Estimates the number of documents in the collection using collection metadata.
@@ -183,7 +182,7 @@ impl<T> Collection<T> {
         &self,
         options: impl Into<Option<EstimatedDocumentCountOptions>>,
     ) -> Result<u64> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .estimated_document_count(options.into()),
         )
@@ -198,7 +197,7 @@ impl<T> Collection<T> {
         filter: impl Into<Option<Document>>,
         options: impl Into<Option<CountOptions>>,
     ) -> Result<u64> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .count_documents(filter.into(), options.into()),
         )
@@ -214,7 +213,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<CountOptions>>,
         session: &mut ClientSession,
     ) -> Result<u64> {
-        RUNTIME.block_on(self.async_collection.count_documents_with_session(
+        runtime::block_on(self.async_collection.count_documents_with_session(
             filter.into(),
             options.into(),
             &mut session.async_client_session,
@@ -227,7 +226,7 @@ impl<T> Collection<T> {
         index: IndexModel,
         options: impl Into<Option<CreateIndexOptions>>,
     ) -> Result<CreateIndexResult> {
-        RUNTIME.block_on(self.async_collection.create_index(index, options))
+        runtime::block_on(self.async_collection.create_index(index, options))
     }
 
     /// Creates the given index on this collection using the provided `ClientSession`.
@@ -237,7 +236,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<CreateIndexOptions>>,
         session: &mut ClientSession,
     ) -> Result<CreateIndexResult> {
-        RUNTIME.block_on(self.async_collection.create_index_with_session(
+        runtime::block_on(self.async_collection.create_index_with_session(
             index,
             options,
             &mut session.async_client_session,
@@ -250,7 +249,7 @@ impl<T> Collection<T> {
         indexes: impl IntoIterator<Item = IndexModel>,
         options: impl Into<Option<CreateIndexOptions>>,
     ) -> Result<CreateIndexesResult> {
-        RUNTIME.block_on(self.async_collection.create_indexes(indexes, options))
+        runtime::block_on(self.async_collection.create_indexes(indexes, options))
     }
 
     /// Creates the given indexes on this collection using the provided `ClientSession`.
@@ -260,7 +259,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<CreateIndexOptions>>,
         session: &mut ClientSession,
     ) -> Result<CreateIndexesResult> {
-        RUNTIME.block_on(self.async_collection.create_indexes_with_session(
+        runtime::block_on(self.async_collection.create_indexes_with_session(
             indexes,
             options,
             &mut session.async_client_session,
@@ -273,7 +272,7 @@ impl<T> Collection<T> {
         query: Document,
         options: impl Into<Option<DeleteOptions>>,
     ) -> Result<DeleteResult> {
-        RUNTIME.block_on(self.async_collection.delete_many(query, options.into()))
+        runtime::block_on(self.async_collection.delete_many(query, options.into()))
     }
 
     /// Deletes all documents stored in the collection matching `query` using the provided
@@ -284,7 +283,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<DeleteOptions>>,
         session: &mut ClientSession,
     ) -> Result<DeleteResult> {
-        RUNTIME.block_on(self.async_collection.delete_many_with_session(
+        runtime::block_on(self.async_collection.delete_many_with_session(
             query,
             options.into(),
             &mut session.async_client_session,
@@ -302,7 +301,7 @@ impl<T> Collection<T> {
         query: Document,
         options: impl Into<Option<DeleteOptions>>,
     ) -> Result<DeleteResult> {
-        RUNTIME.block_on(self.async_collection.delete_one(query, options.into()))
+        runtime::block_on(self.async_collection.delete_one(query, options.into()))
     }
 
     /// Deletes up to one document found matching `query` using the provided `ClientSession`.
@@ -317,7 +316,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<DeleteOptions>>,
         session: &mut ClientSession,
     ) -> Result<DeleteResult> {
-        RUNTIME.block_on(self.async_collection.delete_one_with_session(
+        runtime::block_on(self.async_collection.delete_one_with_session(
             query,
             options.into(),
             &mut session.async_client_session,
@@ -331,7 +330,7 @@ impl<T> Collection<T> {
         filter: impl Into<Option<Document>>,
         options: impl Into<Option<DistinctOptions>>,
     ) -> Result<Vec<Bson>> {
-        RUNTIME.block_on(self.async_collection.distinct(
+        runtime::block_on(self.async_collection.distinct(
             field_name.as_ref(),
             filter.into(),
             options.into(),
@@ -347,7 +346,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<DistinctOptions>>,
         session: &mut ClientSession,
     ) -> Result<Vec<Bson>> {
-        RUNTIME.block_on(self.async_collection.distinct_with_session(
+        runtime::block_on(self.async_collection.distinct_with_session(
             field_name.as_ref(),
             filter.into(),
             options.into(),
@@ -367,7 +366,7 @@ impl<T> Collection<T> {
         update: impl Into<UpdateModifications>,
         options: impl Into<Option<UpdateOptions>>,
     ) -> Result<UpdateResult> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .update_many(query, update.into(), options.into()),
         )
@@ -379,7 +378,7 @@ impl<T> Collection<T> {
         name: impl AsRef<str>,
         options: impl Into<Option<DropIndexOptions>>,
     ) -> Result<()> {
-        RUNTIME.block_on(self.async_collection.drop_index(name, options))
+        runtime::block_on(self.async_collection.drop_index(name, options))
     }
 
     /// Drops the index specified by `name` from this collection using the provided `ClientSession`.
@@ -389,7 +388,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<DropIndexOptions>>,
         session: &mut ClientSession,
     ) -> Result<()> {
-        RUNTIME.block_on(self.async_collection.drop_index_with_session(
+        runtime::block_on(self.async_collection.drop_index_with_session(
             name,
             options,
             &mut session.async_client_session,
@@ -398,7 +397,7 @@ impl<T> Collection<T> {
 
     /// Drops all indexes associated with this collection.
     pub fn drop_indexes(&self, options: impl Into<Option<DropIndexOptions>>) -> Result<()> {
-        RUNTIME.block_on(self.async_collection.drop_indexes(options))
+        runtime::block_on(self.async_collection.drop_indexes(options))
     }
 
     /// Drops all indexes associated with this collection using the provided `ClientSession`.
@@ -407,7 +406,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<DropIndexOptions>>,
         session: &mut ClientSession,
     ) -> Result<()> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .drop_indexes_with_session(options, &mut session.async_client_session),
         )
@@ -418,9 +417,7 @@ impl<T> Collection<T> {
         &self,
         options: impl Into<Option<ListIndexesOptions>>,
     ) -> Result<Cursor<IndexModel>> {
-        RUNTIME
-            .block_on(self.async_collection.list_indexes(options))
-            .map(Cursor::new)
+        runtime::block_on(self.async_collection.list_indexes(options)).map(Cursor::new)
     }
 
     /// Lists all indexes on this collection using the provided `ClientSession`.
@@ -429,17 +426,16 @@ impl<T> Collection<T> {
         options: impl Into<Option<ListIndexesOptions>>,
         session: &mut ClientSession,
     ) -> Result<SessionCursor<IndexModel>> {
-        RUNTIME
-            .block_on(
-                self.async_collection
-                    .list_indexes_with_session(options, &mut session.async_client_session),
-            )
-            .map(SessionCursor::new)
+        runtime::block_on(
+            self.async_collection
+                .list_indexes_with_session(options, &mut session.async_client_session),
+        )
+        .map(SessionCursor::new)
     }
 
     /// Gets the names of all indexes on the collection.
     pub fn list_index_names(&self) -> Result<Vec<String>> {
-        RUNTIME.block_on(self.async_collection.list_index_names())
+        runtime::block_on(self.async_collection.list_index_names())
     }
 
     /// Gets the names of all indexes on the collection using the provided `ClientSession`.
@@ -447,7 +443,7 @@ impl<T> Collection<T> {
         &self,
         session: &mut ClientSession,
     ) -> Result<Vec<String>> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .list_index_names_with_session(&mut session.async_client_session),
         )
@@ -466,7 +462,7 @@ impl<T> Collection<T> {
         options: impl Into<Option<UpdateOptions>>,
         session: &mut ClientSession,
     ) -> Result<UpdateResult> {
-        RUNTIME.block_on(self.async_collection.update_many_with_session(
+        runtime::block_on(self.async_collection.update_many_with_session(
             query,
             update.into(),
             options.into(),
@@ -491,7 +487,7 @@ impl<T> Collection<T> {
         update: impl Into<UpdateModifications>,
         options: impl Into<Option<UpdateOptions>>,
     ) -> Result<UpdateResult> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .update_one(query, update.into(), options.into()),
         )
@@ -516,27 +512,70 @@ impl<T> Collection<T> {
         options: impl Into<Option<UpdateOptions>>,
         session: &mut ClientSession,
     ) -> Result<UpdateResult> {
-        RUNTIME.block_on(self.async_collection.update_one_with_session(
+        runtime::block_on(self.async_collection.update_one_with_session(
             query,
             update.into(),
             options.into(),
             &mut session.async_client_session,
         ))
     }
-}
 
-impl<T> Collection<T>
-where
-    T: DeserializeOwned + Unpin + Send + Sync,
-{
+    /// Starts a new [`ChangeStream`](change_stream/struct.ChangeStream.html) that receives events
+    /// for all changes in this collection. A
+    /// [`ChangeStream`](change_stream/struct.ChangeStream.html) cannot be started on system
+    /// collections.
+    ///
+    /// See the documentation [here](https://docs.mongodb.com/manual/changeStreams/) on change
+    /// streams.
+    ///
+    /// Change streams require either a "majority" read concern or no read concern. Anything else
+    /// will cause a server error.
+    ///
+    /// Also note that using a `$project` stage to remove any of the `_id`, `operationType` or `ns`
+    /// fields will cause an error. The driver requires these fields to support resumability. For
+    /// more information on resumability, see the documentation for
+    /// [`ChangeStream`](change_stream/struct.ChangeStream.html)
+    ///
+    /// If the pipeline alters the structure of the returned events, the parsed type will need to be
+    /// changed via [`ChangeStream::with_type`].
+    pub fn watch(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+    ) -> Result<ChangeStream<ChangeStreamEvent<T>>>
+    where
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
+        runtime::block_on(self.async_collection.watch(pipeline, options)).map(ChangeStream::new)
+    }
+
+    /// Starts a new [`SessionChangeStream`] that receives events for all changes in this collection
+    /// using the provided [`ClientSession`].  See [`Client::watch`](crate::sync::Client::watch) for
+    /// more information.
+    pub fn watch_with_session(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+        session: &mut ClientSession,
+    ) -> Result<SessionChangeStream<ChangeStreamEvent<T>>>
+    where
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
+        runtime::block_on(self.async_collection.watch_with_session(
+            pipeline,
+            options,
+            &mut session.async_client_session,
+        ))
+        .map(SessionChangeStream::new)
+    }
+
     /// Finds the documents in the collection matching `filter`.
     pub fn find(
         &self,
         filter: impl Into<Option<Document>>,
         options: impl Into<Option<FindOptions>>,
     ) -> Result<Cursor<T>> {
-        RUNTIME
-            .block_on(self.async_collection.find(filter.into(), options.into()))
+        runtime::block_on(self.async_collection.find(filter.into(), options.into()))
             .map(Cursor::new)
     }
 
@@ -547,22 +586,26 @@ where
         options: impl Into<Option<FindOptions>>,
         session: &mut ClientSession,
     ) -> Result<SessionCursor<T>> {
-        RUNTIME
-            .block_on(self.async_collection.find_with_session(
-                filter.into(),
-                options.into(),
-                &mut session.async_client_session,
-            ))
-            .map(SessionCursor::new)
+        runtime::block_on(self.async_collection.find_with_session(
+            filter.into(),
+            options.into(),
+            &mut session.async_client_session,
+        ))
+        .map(SessionCursor::new)
     }
+}
 
+impl<T> Collection<T>
+where
+    T: DeserializeOwned + Unpin + Send + Sync,
+{
     /// Finds a single document in the collection matching `filter`.
     pub fn find_one(
         &self,
         filter: impl Into<Option<Document>>,
         options: impl Into<Option<FindOneOptions>>,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .find_one(filter.into(), options.into()),
         )
@@ -576,7 +619,7 @@ where
         options: impl Into<Option<FindOneOptions>>,
         session: &mut ClientSession,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(self.async_collection.find_one_with_session(
+        runtime::block_on(self.async_collection.find_one_with_session(
             filter.into(),
             options.into(),
             &mut session.async_client_session,
@@ -599,7 +642,7 @@ where
         filter: Document,
         options: impl Into<Option<FindOneAndDeleteOptions>>,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .find_one_and_delete(filter, options.into()),
         )
@@ -618,7 +661,7 @@ where
         options: impl Into<Option<FindOneAndDeleteOptions>>,
         session: &mut ClientSession,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(self.async_collection.find_one_and_delete_with_session(
+        runtime::block_on(self.async_collection.find_one_and_delete_with_session(
             filter,
             options.into(),
             &mut session.async_client_session,
@@ -640,7 +683,7 @@ where
         update: impl Into<UpdateModifications>,
         options: impl Into<Option<FindOneAndUpdateOptions>>,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(self.async_collection.find_one_and_update(
+        runtime::block_on(self.async_collection.find_one_and_update(
             filter,
             update.into(),
             options.into(),
@@ -663,7 +706,7 @@ where
         options: impl Into<Option<FindOneAndUpdateOptions>>,
         session: &mut ClientSession,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(self.async_collection.find_one_and_update_with_session(
+        runtime::block_on(self.async_collection.find_one_and_update_with_session(
             filter,
             update.into(),
             options.into(),
@@ -689,7 +732,7 @@ where
         replacement: T,
         options: impl Into<Option<FindOneAndReplaceOptions>>,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(self.async_collection.find_one_and_replace(
+        runtime::block_on(self.async_collection.find_one_and_replace(
             filter,
             replacement,
             options.into(),
@@ -710,7 +753,7 @@ where
         options: impl Into<Option<FindOneAndReplaceOptions>>,
         session: &mut ClientSession,
     ) -> Result<Option<T>> {
-        RUNTIME.block_on(self.async_collection.find_one_and_replace_with_session(
+        runtime::block_on(self.async_collection.find_one_and_replace_with_session(
             filter,
             replacement,
             options.into(),
@@ -734,7 +777,7 @@ where
         docs: impl IntoIterator<Item = impl Borrow<T>>,
         options: impl Into<Option<InsertManyOptions>>,
     ) -> Result<InsertManyResult> {
-        RUNTIME.block_on(self.async_collection.insert_many(docs, options.into()))
+        runtime::block_on(self.async_collection.insert_many(docs, options.into()))
     }
 
     /// Inserts the documents in `docs` into the collection using the provided `ClientSession`.
@@ -749,7 +792,7 @@ where
         options: impl Into<Option<InsertManyOptions>>,
         session: &mut ClientSession,
     ) -> Result<InsertManyResult> {
-        RUNTIME.block_on(self.async_collection.insert_many_with_session(
+        runtime::block_on(self.async_collection.insert_many_with_session(
             docs,
             options.into(),
             &mut session.async_client_session,
@@ -767,7 +810,7 @@ where
         doc: impl Borrow<T>,
         options: impl Into<Option<InsertOneOptions>>,
     ) -> Result<InsertOneResult> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_collection
                 .insert_one(doc.borrow(), options.into()),
         )
@@ -785,7 +828,7 @@ where
         options: impl Into<Option<InsertOneOptions>>,
         session: &mut ClientSession,
     ) -> Result<InsertOneResult> {
-        RUNTIME.block_on(self.async_collection.insert_one_with_session(
+        runtime::block_on(self.async_collection.insert_one_with_session(
             doc.borrow(),
             options.into(),
             &mut session.async_client_session,
@@ -804,7 +847,7 @@ where
         replacement: impl Borrow<T>,
         options: impl Into<Option<ReplaceOptions>>,
     ) -> Result<UpdateResult> {
-        RUNTIME.block_on(self.async_collection.replace_one(
+        runtime::block_on(self.async_collection.replace_one(
             query,
             replacement.borrow(),
             options.into(),
@@ -825,7 +868,7 @@ where
         options: impl Into<Option<ReplaceOptions>>,
         session: &mut ClientSession,
     ) -> Result<UpdateResult> {
-        RUNTIME.block_on(self.async_collection.replace_one_with_session(
+        runtime::block_on(self.async_collection.replace_one_with_session(
             query,
             replacement.borrow(),
             options.into(),
