@@ -1,8 +1,9 @@
 use std::fmt::Debug;
 
-use super::{ClientSession, Collection, Cursor, SessionCursor};
+use super::{ChangeStream, ClientSession, Collection, Cursor, SessionChangeStream, SessionCursor};
 use crate::{
     bson::Document,
+    change_stream::{event::ChangeStreamEvent, options::ChangeStreamOptions},
     error::Result,
     options::{
         AggregateOptions,
@@ -15,8 +16,8 @@ use crate::{
         WriteConcern,
     },
     results::CollectionSpecification,
+    runtime,
     Database as AsyncDatabase,
-    RUNTIME,
 };
 
 /// `Database` is the client-side abstraction of a MongoDB database. It can be used to perform
@@ -107,7 +108,7 @@ impl Database {
 
     /// Drops the database, deleting all data, collections, users, and indexes stored in it.
     pub fn drop(&self, options: impl Into<Option<DropDatabaseOptions>>) -> Result<()> {
-        RUNTIME.block_on(self.async_database.drop(options.into()))
+        runtime::block_on(self.async_database.drop(options.into()))
     }
 
     /// Drops the database, deleting all data, collections, users, and indexes stored in it using
@@ -117,7 +118,7 @@ impl Database {
         options: impl Into<Option<DropDatabaseOptions>>,
         session: &mut ClientSession,
     ) -> Result<()> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_database
                 .drop_with_session(options.into(), &mut session.async_client_session),
         )
@@ -130,12 +131,11 @@ impl Database {
         filter: impl Into<Option<Document>>,
         options: impl Into<Option<ListCollectionsOptions>>,
     ) -> Result<Cursor<CollectionSpecification>> {
-        RUNTIME
-            .block_on(
-                self.async_database
-                    .list_collections(filter.into(), options.into()),
-            )
-            .map(Cursor::new)
+        runtime::block_on(
+            self.async_database
+                .list_collections(filter.into(), options.into()),
+        )
+        .map(Cursor::new)
     }
 
     /// Gets information about each of the collections in the database using the provided
@@ -147,13 +147,12 @@ impl Database {
         options: impl Into<Option<ListCollectionsOptions>>,
         session: &mut ClientSession,
     ) -> Result<SessionCursor<CollectionSpecification>> {
-        RUNTIME
-            .block_on(self.async_database.list_collections_with_session(
-                filter.into(),
-                options.into(),
-                &mut session.async_client_session,
-            ))
-            .map(SessionCursor::new)
+        runtime::block_on(self.async_database.list_collections_with_session(
+            filter.into(),
+            options.into(),
+            &mut session.async_client_session,
+        ))
+        .map(SessionCursor::new)
     }
 
     /// Gets the names of the collections in the database.
@@ -161,7 +160,7 @@ impl Database {
         &self,
         filter: impl Into<Option<Document>>,
     ) -> Result<Vec<String>> {
-        RUNTIME.block_on(self.async_database.list_collection_names(filter.into()))
+        runtime::block_on(self.async_database.list_collection_names(filter.into()))
     }
 
     /// Gets the names of the collections in the database using the provided `ClientSession`.
@@ -170,7 +169,7 @@ impl Database {
         filter: impl Into<Option<Document>>,
         session: &mut ClientSession,
     ) -> Result<Vec<String>> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_database.list_collection_names_with_session(
                 filter.into(),
                 &mut session.async_client_session,
@@ -187,7 +186,7 @@ impl Database {
         name: impl AsRef<str>,
         options: impl Into<Option<CreateCollectionOptions>>,
     ) -> Result<()> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_database
                 .create_collection(name.as_ref(), options.into()),
         )
@@ -204,7 +203,7 @@ impl Database {
         options: impl Into<Option<CreateCollectionOptions>>,
         session: &mut ClientSession,
     ) -> Result<()> {
-        RUNTIME.block_on(self.async_database.create_collection_with_session(
+        runtime::block_on(self.async_database.create_collection_with_session(
             name.as_ref(),
             options.into(),
             &mut session.async_client_session,
@@ -221,7 +220,7 @@ impl Database {
         command: Document,
         selection_criteria: impl Into<Option<SelectionCriteria>>,
     ) -> Result<Document> {
-        RUNTIME.block_on(
+        runtime::block_on(
             self.async_database
                 .run_command(command, selection_criteria.into()),
         )
@@ -238,7 +237,7 @@ impl Database {
         selection_criteria: impl Into<Option<SelectionCriteria>>,
         session: &mut ClientSession,
     ) -> Result<Document> {
-        RUNTIME.block_on(self.async_database.run_command_with_session(
+        runtime::block_on(self.async_database.run_command_with_session(
             command,
             selection_criteria.into(),
             &mut session.async_client_session,
@@ -255,9 +254,7 @@ impl Database {
         options: impl Into<Option<AggregateOptions>>,
     ) -> Result<Cursor<Document>> {
         let pipeline: Vec<Document> = pipeline.into_iter().collect();
-        RUNTIME
-            .block_on(self.async_database.aggregate(pipeline, options.into()))
-            .map(Cursor::new)
+        runtime::block_on(self.async_database.aggregate(pipeline, options.into())).map(Cursor::new)
     }
 
     /// Runs an aggregation operation using the provided `ClientSession`.
@@ -271,12 +268,52 @@ impl Database {
         session: &mut ClientSession,
     ) -> Result<SessionCursor<Document>> {
         let pipeline: Vec<Document> = pipeline.into_iter().collect();
-        RUNTIME
-            .block_on(self.async_database.aggregate_with_session(
-                pipeline,
-                options.into(),
-                &mut session.async_client_session,
-            ))
-            .map(SessionCursor::new)
+        runtime::block_on(self.async_database.aggregate_with_session(
+            pipeline,
+            options.into(),
+            &mut session.async_client_session,
+        ))
+        .map(SessionCursor::new)
+    }
+
+    /// Starts a new [`ChangeStream`](change_stream/struct.ChangeStream.html) that receives events
+    /// for all changes in this database. The stream does not observe changes from system
+    /// collections and cannot be started on "config", "local" or "admin" databases.
+    ///
+    /// See the documentation [here](https://docs.mongodb.com/manual/changeStreams/) on change
+    /// streams.
+    ///
+    /// Change streams require either a "majority" read concern or no read
+    /// concern. Anything else will cause a server error.
+    ///
+    /// Note that using a `$project` stage to remove any of the `_id`, `operationType` or `ns`
+    /// fields will cause an error. The driver requires these fields to support resumability. For
+    /// more information on resumability, see the documentation for
+    /// [`ChangeStream`](change_stream/struct.ChangeStream.html)
+    ///
+    /// If the pipeline alters the structure of the returned events, the parsed type will need to be
+    /// changed via [`ChangeStream::with_type`].
+    pub fn watch(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+    ) -> Result<ChangeStream<ChangeStreamEvent<Document>>> {
+        runtime::block_on(self.async_database.watch(pipeline, options)).map(ChangeStream::new)
+    }
+
+    /// Starts a new [`SessionChangeStream`] that receives events for all changes in this database
+    /// using the provided [`ClientSession`].  See [`Database::watch`] for more information.
+    pub fn watch_with_session(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+        session: &mut ClientSession,
+    ) -> Result<SessionChangeStream<ChangeStreamEvent<Document>>> {
+        runtime::block_on(self.async_database.watch_with_session(
+            pipeline,
+            options,
+            &mut session.async_client_session,
+        ))
+        .map(SessionChangeStream::new)
     }
 }
